"""Windows 鼠标指针配置解析"""

from pathlib import Path
from typing import (
    Any,
    Literal,
    Protocol,
    TypedDict,
    cast,
)

from ani2xcur.config_parse.parse import (
    ParsedINF,
    parse_inf_file,
)


CursorShemeINF = TypedDict(
    "CursorShemeINF",
    {
        "Version": dict[str, str | list[str]],  # 版本信息
        "DefaultInstall": dict[str, list[str]],  # 默认需要安装的配置
        "DestinationDirs": dict[str, str | list[str]],  # 目的路径
        "Scheme.Reg": list[str],  # 鼠标指针方案在注册表中的存储位置
        "Wreg": list[str],  # 当前应用的鼠标指针设置
        "Scheme.Cur": list[str],  # 需要复制到系统中的光标文件名列表
        "Strings": dict[str, str],  # 配置中使用的变量
    },
    total=False,
)
"""预处理后的鼠标指针配置 INF 结构"""

INFSectionName = Literal[
    "Version",  # 版本信息
    "DefaultInstall",  # 默认需要安装的配置
    "DestinationDirs",  # 目的路径
    "Scheme.Reg",  # 鼠标指针方案在注册表中的存储位置
    "Scheme.Cur",  # 需要复制到系统中的光标文件名列表
    "Wreg",  # 当前应用的鼠标指针设置
    "Strings",  # 配置中使用的变量
]
"""INF 文件已知键名"""


class INFSectionDict(Protocol):
    """INF 选项结构字典"""

    def get(  # pylint: disable=missing-function-docstring
        self,
        key: Literal["var", "constant"],
        default: Any = ...,
    ) -> str | dict[str, str | list[str]] | Any: ...


class KnownINFSections(Protocol):
    """已知的 INF 结构"""

    def __getitem__(
        self,
        key: INFSectionName,
    ) -> INFSectionDict: ...
    def __contains__(
        self,
        key: object,
    ) -> bool: ...


def preprocess_inf_to_cursor_scheme(
    parsed: ParsedINF,
) -> CursorShemeINF:
    """将 ParsedINF 处理为鼠标指针已知键的扁平结构

    返回包含常见节的字典:
    - Version: 该节的 var & constant 原样返回 (若存在)
    - DefaultInstall: 各键保证为 list[str]
    - DestinationDirs: 直接返回 var 映射 (值可能为 str 或 list[str])
    - Scheme.Reg: Scheme.Reg 节的常量行列表
    - Wreg: Wreg 节的常量行列表
    - Scheme.Cur: Scheme.Cur 节的常量行列表
    - Strings: Strings 节的 var 映射

    Args:
        parsed (ParsedINF): INF 类型字典
    Returns:
        CursorShemeINF: 预处理后的鼠标指针 INF 信息字典
    Raises:
        ValueError: 当鼠标指针配置文件不完整时
    """

    def _ensure_list(
        v: str | list[str],
    ) -> list[str]:
        return v if isinstance(v, list) else [v]

    out: CursorShemeINF = {}
    parsed_known = cast(KnownINFSections, parsed)

    if "Version" in parsed_known:
        # 合并 var 和 constant（如果需要可扩展）
        out["Version"] = parsed_known["Version"].get("var", {})

    if "DefaultInstall" in parsed_known:
        di: dict[str, list[str]] = {}
        for k, v in parsed_known["DefaultInstall"].get("var", {}).items():
            di[k] = _ensure_list(v)
        out["DefaultInstall"] = di

    if "DestinationDirs" in parsed_known:
        out["DestinationDirs"] = parsed_known["DestinationDirs"].get("var", {})

    # 一些节在不同 INF 中可能为 var 或 constant，这里以常见样式提取
    if "Scheme.Reg" in parsed_known:
        out["Scheme.Reg"] = parsed_known["Scheme.Reg"].get("constant", [])
    else:
        raise ValueError("未找到 Scheme.Reg 键, 鼠标指针配置不完整")

    if "Wreg" in parsed_known:
        out["Wreg"] = parsed_known["Wreg"].get("constant", [])

    if "Scheme.Cur" in parsed_known:
        out["Scheme.Cur"] = parsed_known["Scheme.Cur"].get("constant", [])
    else:
        raise ValueError("未找到 Scheme.Cur 键, 鼠标指针配置不完整")

    if "Strings" in parsed_known:
        out["Strings"] = parsed_known["Strings"].get("var", {})
    else:
        raise ValueError("未找到 Strings 键, 鼠标指针配置不完整")

    return out


def parse_inf_file_content(
    inf_path: Path,
) -> CursorShemeINF:
    """从 INF 文件中获取鼠标指针配置数据

    Args:
        inf_path (Path): 鼠标指针的 INF 配置文件
    Returns:
        CursorShemeINF: 鼠标指针配置数据
    Raises:
        ValueError: 当鼠标指针配置文件不完整时
    """
    inf = parse_inf_file(inf_path)
    try:
        return preprocess_inf_to_cursor_scheme(inf)
    except ValueError as e:
        raise e


def dict_to_inf_strings_format(
    data_dict: dict[str, str],
    indent_width: int | None = 12,
) -> str:
    """
    将字典转换为 INF 文件 [Strings] 部分的格式

    Args:
        data_dict (dict[str, str]): 包含键值对的字典
        indent_width (int | None): 缩进宽度, 默认为 12 个空格

    Returns:
        str: 格式化的字符串
    """
    lines = []
    for key, value in data_dict.items():
        # 使用制表符或空格对齐，保持与原格式一致
        separator = "\t\t" if len(key) < indent_width else "\t"
        line = f'{key}{separator}= "{value}"'
        lines.append(line)

    return "\n".join(lines)



"""管理工具基础配置"""

from pathlib import Path
from typing import (
    TypeAlias,
    TypedDict,
)


WIN_CURSOR_KEYS = [
    "Arrow",  # 正常选择
    "Help",  # 帮助选择
    "AppStarting",  # 后台运行
    "Wait",  # 忙
    "Crosshair",  # 精确选择
    "IBeam",  # 文本选择
    "NWPen",  # 手写
    "No",  # 不可用
    "SizeNS",  # 垂直调整大小
    "SizeWE",  # 水平调整大小
    "SizeNWSE",  # 沿对角线调整大小 1
    "SizeNESW",  # 沿对角线调整大小 2
    "SizeAll",  # 移动
    "UpArrow",  # 候选
    "Hand",  # 链接选择
    "Pin",  # 位置选择
    "Person",  # 个人选择
]
"""Windows 鼠标指针对应的键列表"""

LINUX_CURSOR_KEYS = [
    "left_ptr",  # 正常选择
    "question_arrow",  # 帮助选择
    "left_ptr_watch",  # 后台运行
    "wait",  # 忙
    "cross",  # 精确选择
    "xterm",  # 文本选择
    "pencil",  # 手写
    "circle",  # 不可用
    "bottom_side",  # 垂直调整大小
    "left_side",  # 水平调整大小
    "bottom_right_corner",  # 沿对角线调整大小 1
    "bottom_left_corner",  # 沿对角线调整大小 2
    "move",  # 移动
    "dotbox",  # 候选
    "hand2",  # 链接选择
    "center_ptr",  # 位置选择
    "right_ptr",  # 个人选择
]
"""Linux 鼠标指针对应的键列表"""


class CursorKeys(TypedDict):
    """鼠标指针键名表"""

    win: list[str]
    """Windows 鼠标指针对应的键列表"""

    linux: list[str]
    """Linux 鼠标指针对应的键列表"""


CURSOR_KEYS: CursorKeys = {"win": WIN_CURSOR_KEYS, "linux": LINUX_CURSOR_KEYS}
"""鼠标指针对应的键值表 (Windows / Linux)"""


class CursorFilePair(TypedDict):
    """单个光标文件的源路径与目标路径"""

    dst_path: Path | None
    """复制到系统的目标路径"""

    src_path: Path | None
    """原始文件路径"""


class KnownCursorMap(TypedDict, total=False):
    """Windows / Linux 标准鼠标指针类型, 键名以 Windows 中的作为标准"""

    Arrow: CursorFilePair | None
    """正常选择 (Arrow <-> left_ptr)"""

    Help: CursorFilePair | None
    """帮助选择 (Help <-> question_arrow)"""

    AppStarting: CursorFilePair | None
    """后台运行 (AppStarting <-> left_ptr_watch)"""

    Wait: CursorFilePair | None
    """忙 (Wait <-> wait)"""

    Crosshair: CursorFilePair | None
    """精确选择 (Crosshair <-> cross)"""

    IBeam: CursorFilePair | None
    """文本选择 (IBeam <-> xterm)"""

    NWPen: CursorFilePair | None
    """手写 (NWPen <-> pencil)"""

    No: CursorFilePair | None
    """不可用 (No <-> circle)"""

    SizeNS: CursorFilePair | None
    """垂直调整大小 (SizeNS <-> bottom_side)"""

    SizeWE: CursorFilePair | None
    """水平调整大小 (SizeWE <-> left_side)"""

    SizeNWSE: CursorFilePair | None
    """沿对角线调整大小 1 (SizeNWSE <-> bottom_right_corner)"""

    SizeNESW: CursorFilePair | None
    """沿对角线调整大小 2 (SizeNESW <-> bottom_left_corner)"""

    SizeAll: CursorFilePair | None
    """移动 (SizeAll <-> move)"""

    UpArrow: CursorFilePair | None
    """候选 (UpArrow <-> dotbox)"""

    Hand: CursorFilePair | None
    """链接选择 (Hand <-> hand2)"""

    Pin: CursorFilePair | None
    """位置选择 (Pin <-> center_ptr)"""

    Person: CursorFilePair | None
    """个人选择 (Person <-> right_ptr)"""


CursorMap: TypeAlias = dict[str, CursorFilePair] | KnownCursorMap
"""鼠标指针类型与对应的路径地图"""


class WinCursorsConfig(TypedDict, total=False):
    """Windows 鼠标指针配置信息"""

    Arrow: str | None
    """正常选择 (Arrow -> left_ptr)"""

    Help: str | None
    """帮助选择 (Help -> question_arrow)"""

    AppStarting: str | None
    """后台运行 (AppStarting -> left_ptr_watch)"""

    Wait: str | None
    """忙 (Wait -> wait)"""

    Crosshair: str | None
    """精确选择 (Crosshair -> cross)"""

    IBeam: str | None
    """文本选择 (IBeam -> xterm)"""

    NWPen: str | None
    """手写 (NWPen -> pencil)"""

    No: str | None
    """不可用 (No -> circle)"""

    SizeNS: str | None
    """垂直调整大小 (SizeNS -> bottom_side)"""

    SizeWE: str | None
    """水平调整大小 (SizeWE -> left_side)"""

    SizeNWSE: str | None
    """沿对角线调整大小 1 (SizeNWSE -> bottom_right_corner)"""

    SizeNESW: str | None
    """沿对角线调整大小 2 (SizeNESW -> bottom_left_corner)"""

    SizeAll: str | None
    """移动 (SizeAll -> move)"""

    UpArrow: str | None
    """候选 (UpArrow -> dotbox)"""

    Hand: str | None
    """链接选择 (Hand -> hand2)"""

    Pin: str | None
    """位置选择 (Pin -> center_ptr)"""

    Person: str | None
    """个人选择 (Person -> right_ptr)"""


class LinuxCursorsConfig(TypedDict, total=False):
    """Linux 标准鼠标指针类型"""

    left_ptr: str | None
    """正常选择 (left_ptr -> Arrow)"""

    question_arrow: str | None
    """帮助选择 (question_arrow -> Help)"""

    left_ptr_watch: str | None
    """后台运行 (left_ptr_watch -> AppStarting)"""

    wait: str | None
    """忙 (wait -> Wait)"""

    cross: str | None
    """精确选择 (cross -> Crosshair)"""

    xterm: str | None
    """文本选择 (xterm -> IBeam)"""

    pencil: str | None
    """手写 (pencil -> NWPen)"""

    circle: str | None
    """不可用 (circle -> No)"""

    bottom_side: str | None
    """垂直调整大小 (bottom_side -> SizeNS)"""

    left_side: str | None
    """水平调整大小 (left_side -> SizeWE)"""

    bottom_right_corner: str | None
    """沿对角线调整大小 1 (bottom_right_corner -> SizeNWSE)"""

    bottom_left_corner: str | None
    """沿对角线调整大小 2 (bottom_left_corner -> SizeNESW)"""

    move: str | None
    """移动 (move -> SizeAll)"""

    dotbox: str | None
    """候选 (dotbox -> UpArrow)"""

    hand2: str | None
    """链接选择 (hand2 -> Hand)"""

    center_ptr: str | None
    """位置选择 (center_ptr -> Pin)"""

    right_ptr: str | None
    """个人选择 (right_ptr -> Person)"""


LINUX_CURSOR_LINKS = [
    # left_ptr
    ["left_ptr", "context-menu"],
    ["left_ptr", "grabbing"],
    ["left_ptr", "hand1"],
    ["left_ptr", "arrow"],
    ["left_ptr", "closedhand"],
    ["left_ptr", "default"],
    ["left_ptr", "dnd-none"],
    ["left_ptr", "grab"],
    ["left_ptr", "openhand"],
    ["left_ptr", "top_left_arrow"],
    ["left_ptr", "fcf21c00b30f7e3f83fe0dfd12e71cff"],
    # left_ptr_watch
    ["left_ptr_watch", "progress"],
    ["left_ptr_watch", "00000000000000020006000e7e9ffc3f"],
    ["left_ptr_watch", "08e8e1c95fe2fc01f976f1e063a24ccd"],
    ["left_ptr_watch", "3ecb610c1bf2410f44200f48c40d3599"],
    # right_ptr
    ["right_ptr", "draft_large"],
    ["right_ptr", "draft_small"],
    ["right_ptr", "e-resize"],
    # move
    ["move", "all-scroll"],
    ["move", "fleur"],
    ["move", "size_all"],
    ["move", "4498f0e0c1937ffe01fd06f973665830"],
    ["move", "9081237383d90e509aa00f00170e968f"],
    # question_arrow
    ["question_arrow", "dnd-ask"],
    ["question_arrow", "help"],
    ["question_arrow", "left_ptr_help"],
    ["question_arrow", "whats_this"],
    ["question_arrow", "5c6cd98b3f3ebcb1f9c7f1c204630408"],
    ["question_arrow", "d9ce0ab605698f320427677b458ad60b"],
    # xterm
    ["xterm", "ibeam"],
    ["xterm", "text"],
    # wait
    ["wait", "watch"],
    # hand2
    ["hand2", "pointer"],
    ["hand2", "pointing_hand"],
    ["hand2", "9d800788f1b08800ae810202380a0822"],
    ["hand2", "e29285e634086352946a0e7090d73106"],
    # pencil
    ["pencil", "copy"],
    ["pencil", "dnd-copy"],
    ["pencil", "dnd-move"],
    ["pencil", "dnd-link"],
    ["pencil", "link"],
    ["pencil", "pointer-move"],
    ["pencil", "alias"],
    ["pencil", "draft"],
    ["pencil", "1081e37283d90000800003c07f3ef6bf"],
    ["pencil", "3085a0e285430894940527032f8b26df"],
    ["pencil", "6407b0e94181790501fd1e167b474872"],
    ["pencil", "640fb0e74195791501fd1ed57b41487f"],
    ["pencil", "a2a266d0498c3104214a47bd64ab0fc8"],
    ["pencil", "b66166c04f8c3109214a4fbd64a50fc8"],
    # circle
    ["circle", "crossed_circle"],
    ["circle", "dnd_no_drop"],
    ["circle", "X_cursor"],
    ["circle", "x-cursor"],
    ["circle", "forbidden"],
    ["circle", "no-drop"],
    ["circle", "not-allowed"],
    ["circle", "pirate"],
    ["circle", "03b6e0fcb3499374a867c041f52298f0"],
    # cross
    ["cross", "crosshair"],
    ["cross", "tcross"],
    ["cross", "color-picker"],
    ["cross", "cross_reverse"],
    ["cross", "diamond_cross"],
    # bottom_left_corner
    ["bottom_left_corner", "fd_double_arrow"],
    ["bottom_left_corner", "ll_angle"],
    ["bottom_left_corner", "top_right_corner"],
    ["bottom_left_corner", "ur_angle"],
    ["bottom_left_corner", "ne-resize"],
    ["bottom_left_corner", "nesw-resize"],
    ["bottom_left_corner", "size_bdiag"],
    ["bottom_left_corner", "sw-resize"],
    ["bottom_left_corner", "fcf1c3c7cd4491d801f1e1c78f100000"],
    # bottom_right_corner
    ["bottom_right_corner", "bd_double_arrow"],
    ["bottom_right_corner", "lr_angle"],
    ["bottom_right_corner", "top_left_corner"],
    ["bottom_right_corner", "ul_angle"],
    ["bottom_right_corner", "nw-resize"],
    ["bottom_right_corner", "nwse-resize"],
    ["bottom_right_corner", "se-resize"],
    ["bottom_right_corner", "size_fdiag"],
    ["bottom_right_corner", "c7088f0f3e6c8088236ef8e1e3e70000"],
    # bottom_side
    ["bottom_side", "bottom_tee"],
    ["bottom_side", "plus"],
    ["bottom_side", "sb_down_arrow"],
    ["bottom_side", "sb_up_arrow"],
    ["bottom_side", "sb_v_double_arrow"],
    ["bottom_side", "top_side"],
    ["bottom_side", "top_tee"],
    ["bottom_side", "cell"],
    ["bottom_side", "double_arrow"],
    ["bottom_side", "down-arrow"],
    ["bottom_side", "n-resize"],
    ["bottom_side", "ns-resize"],
    ["bottom_side", "row-resize"],
    ["bottom_side", "s-resize"],
    ["bottom_side", "up-arrow"],
    ["bottom_side", "v_double_arrow"],
    ["bottom_side", "size_ver"],
    ["bottom_side", "00008160000006810000408080010102"],
    ["bottom_side", "2870a09082c31050810ffdffffe0204"],
    # left_side
    ["left_side", "left_tee"],
    ["left_side", "right_side"],
    ["left_side", "right_tee"],
    ["left_side", "sb_h_double_arrow"],
    ["left_side", "sb_left_arrow"],
    ["left_side", "sb_right_arrow"],
    ["left_side", "col-resize"],
    ["left_side", "ew-resize"],
    ["left_side", "h_double_arrow"],
    ["left_side", "left-arrow"],
    ["left_side", "right-arrow"],
    ["left_side", "size-hor"],
    ["left_side", "size-ver"],
    ["left_side", "split_h"],
    ["left_side", "split_v"],
    ["left_side", "w-resize"],
    ["left_side", "size_hor"],
    ["left_side", "028006030e0e7ebffc7f7070c0600140"],
    ["left_side", "14fef782d02440884392942c1120523"],
    # dotbox
    ["dotbox", "dot_box_mask"],
    ["dotbox", "draped_box"],
    ["dotbox", "icon"],
    ["dotbox", "target"],
]
"""Linux 鼠标指针需要链接的别名"""


class LocalCursor(TypedDict):
    """本地已安装的鼠标指针配置"""

    name: str
    """鼠标指针配置名称"""

    cursor_files: list[Path]
    """鼠标指针文件列表"""

    install_paths: list[Path]
    """鼠标指针安装路径列表 (鼠标指针文件所在父路径)"""


CursorSchemesList: TypeAlias = list[LocalCursor]
"""本地已安装的鼠标指针配置列表"""

LINUX_ICONS_PATH = Path("/usr/share/icons")
"""Linux 图标目录"""

LINUX_USER_ICONS_PATH = Path("~/.icons").expanduser()
"""Linux 用户图标目录"""


class CurrentCursorInfo(TypedDict):
    """当前桌面平台使用的鼠标指针名称和大小信息"""

    platform: str
    """桌面平台"""

    cursor_name: str
    """鼠标指针名称"""

    cursor_size: int
    """鼠标指针大小"""


CurrentCursorInfoList: TypeAlias = list[CurrentCursorInfo]
"""当前桌面平台使用的鼠标指针名称和大小信息列表"""

WINDOWS_USER_CURSOR_PATH = Path("~").expanduser() / "AppData" / "Local" / "Cursors"
"""Windows 系统安装鼠标的用户目录路径"""



"""鼠标指针转换工具"""

import sys
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Annotated

import typer
import click

from ani2xcur.cursor_conversion.convert import (
    win_cursor_to_x11,
    x11_cursor_to_win,
)
from ani2xcur.cursor_conversion.win2xcur_warp import (
    Win2xcurArgs,
    X2wincurArgs,
)
from ani2xcur.smart_finder import (
    find_inf_file,
    find_desktop_entry_file,
)
from ani2xcur.config import (
    LOGGER_NAME,
    LOGGER_LEVEL,
    LOGGER_COLOR,
    SMART_FINDER_SEARCH_DEPTH,
)
from ani2xcur.logger import get_logger
from ani2xcur.file_operations.archive_manager import (
    create_archive,
    SUPPORTED_ARCHIVE_FORMAT,
)
from ani2xcur.manager.image_magick_manager import check_image_magick_is_installed
from ani2xcur.utils import is_http_or_https

logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


def win2xcur(
    input_path: Annotated[
        str,
        typer.Argument(
            help="Windows 鼠标指针文件的路径, 可以为 inf / ani / cur 文件路径, 或者鼠标指针压缩包文件路径, 也可以是鼠标指针压缩包的下载链接",
        ),
    ],
    output_path: Annotated[
        Path | None,
        typer.Option(
            help="保存转换后的鼠标指针路径",
            resolve_path=True,
        ),
    ] = None,
    shadow: Annotated[
        bool,
        typer.Option(
            help="是否模拟 Windows 的阴影效果",
        ),
    ] = False,
    shadow_opacity: Annotated[
        int,
        typer.Option(
            help="阴影的不透明度 (0 到 255)",
            min=0,
            max=255,
        ),
    ] = 50,
    shadow_radius: Annotated[
        float,
        typer.Option(
            help="阴影模糊效果的半径 (宽度的分数值)",
        ),
    ] = 0.1,
    shadow_sigma: Annotated[
        float,
        typer.Option(
            help="阴影模糊效果的西格玛值 (宽度的分数值)",
        ),
    ] = 0.1,
    shadow_x: Annotated[
        float,
        typer.Option(
            help="阴影的 x 偏移量 (宽度的分数值)",
        ),
    ] = 0.05,
    shadow_y: Annotated[
        float,
        typer.Option(
            help="阴影的 y 偏移量 (高度的分数值)",
        ),
    ] = 0.05,
    shadow_color: Annotated[
        str,
        typer.Option(
            help="阴影的颜色 (十六进制颜色格式)",
        ),
    ] = "#000000",
    scale: Annotated[
        float | None,
        typer.Option(
            help="按指定倍数缩放光标",
        ),
    ] = None,
    compress: Annotated[
        bool,
        typer.Option(
            help="转换完成后将鼠标指针打包成压缩包",
        ),
    ] = False,
    compress_format: Annotated[
        str,
        typer.Option(
            help="打包成压缩包时使用的压缩包格式",
            click_type=click.Choice(SUPPORTED_ARCHIVE_FORMAT),
        ),
    ] = ".zip",
) -> None:
    """将 Windows 鼠标指针文件包转换为 Linux 鼠标指针文件包"""
    if not check_image_magick_is_installed():
        logger.error("未安装 ImageMagick, 无法进行鼠标指针转换, 请使用 ani2xcur imagemagick install 命令进行安装")
        sys.exit(1)

    logger.info("将 '%s' 的 Windows 鼠标指针主题包转换为 Linux 鼠标指针主题包中", input_path)

    win2x_args: Win2xcurArgs = {
        "shadow": shadow,
        "shadow_opacity": shadow_opacity,
        "shadow_radius": shadow_radius,
        "shadow_sigma": shadow_sigma,
        "shadow_x": shadow_x,
        "shadow_y": shadow_y,
        "shadow_color": shadow_color,
        "scale": scale,
    }

    if output_path is None:
        if is_http_or_https(input_path):
            # 如果是 URL, 默认保存在当前执行目录下
            output_path = Path.cwd()
        else:
            # 如果是本地路径, 保存在输入文件的父目录
            output_path = Path(input_path).resolve().parent

    with TemporaryDirectory() as temp_dir:
        temp_dir = Path(temp_dir)
        inf_file = find_inf_file(
            input_file=input_path,
            temp_dir=temp_dir,
            depth=SMART_FINDER_SEARCH_DEPTH,
        )

        if inf_file is None:
            logger.error("未找到鼠标指针的 INF 配置文件路径")
            sys.exit(1)

        save_path = win_cursor_to_x11(
            inf_file=inf_file,
            output_path=output_path,
            win2x_args=win2x_args,
        )

    logger.info("鼠标指针转换完成, 保存路径: '%s'", save_path)
    if compress:
        logger.info("将鼠标指针进行打包")
        archive_path = output_path / f"{save_path.name}{compress_format}"
        create_archive(
            sources=save_path,
            archive_path=archive_path,
        )
        logger.info("鼠标指针打包完成, 保存路径: '%s'", archive_path)


def x2wincur(
    input_path: Annotated[
        str,
        typer.Argument(
            help="Linux 鼠标指针文件的路径, 可以为 index.theme 文件路径, 或者鼠标指针压缩包文件路径, 也可以是鼠标指针压缩包的下载链接",
        ),
    ],
    output_path: Annotated[
        Path | None,
        typer.Option(
            help="保存转换后的鼠标指针路径",
            resolve_path=True,
        ),
    ] = None,
    scale: Annotated[
        float | None,
        typer.Option(
            help="按指定倍数缩放光标",
        ),
    ] = None,
    compress: Annotated[
        bool,
        typer.Option(
            help="转换完成后将鼠标指针打包成压缩包",
        ),
    ] = False,
    compress_format: Annotated[
        str,
        typer.Option(
            help="打包成压缩包时使用的压缩包格式",
            click_type=click.Choice(SUPPORTED_ARCHIVE_FORMAT),
        ),
    ] = ".zip",
) -> None:
    """将 Linux 鼠标指针文件包转换为 Windows 鼠标指针文件包"""
    if not check_image_magick_is_installed():
        logger.error("未安装 ImageMagick, 无法进行鼠标指针转换, 请使用 ani2xcur imagemagick install 命令进行安装")
        sys.exit(1)

    logger.info("将 '%s' 的 Linux 鼠标指针主题包转换为 Windows 鼠标指针主题包中", input_path)
    x2win_args: X2wincurArgs = {
        "scale": scale,
    }

    if output_path is None:
        if is_http_or_https(input_path):
            # 如果是 URL, 默认保存在当前执行目录下
            output_path = Path.cwd()
        else:
            # 如果是本地路径, 保存在输入文件的父目录
            output_path = Path(input_path).resolve().parent

    with TemporaryDirectory() as temp_dir:
        temp_dir = Path(temp_dir)
        desktop_entry_file = find_desktop_entry_file(
            input_file=input_path,
            temp_dir=temp_dir,
            depth=SMART_FINDER_SEARCH_DEPTH,
        )

        if desktop_entry_file is None:
            logger.error("未找到鼠标指针的 DesktopEntry 配置文件路径")
            sys.exit(1)

        if not (desktop_entry_file.parent / "cursors").is_dir():
            logger.error("鼠标指针目录缺失, 无法进行鼠标指针转换")
            sys.exit(1)

        save_path = x11_cursor_to_win(
            desktop_entry_file=desktop_entry_file,
            output_path=output_path,
            x2win_args=x2win_args,
        )

    logger.info("鼠标指针转换完成, 保存路径: '%s'", save_path)
    if compress:
        logger.info("将鼠标指针进行打包")
        archive_path = output_path / f"{save_path.name}{compress_format}"
        create_archive(
            sources=save_path,
            archive_path=archive_path,
        )
        logger.info("鼠标指针打包完成, 保存路径: '%s'", archive_path)



"""其他工具合集"""

import ctypes
import gc
import getpass
import random
import string
import sys
from typing import Any
from pathlib import Path
from urllib.parse import urlparse

from ani2xcur.logger import get_logger
from ani2xcur.config import (
    LOGGER_COLOR,
    LOGGER_LEVEL,
    LOGGER_NAME,
)


logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


def save_convert_to_float(
    value: Any,
) -> float | Any:
    """尝试将值转换为浮点数

    Args:
        value (Any): 用于转换的值
    Returns:
        (float | Any): 转换成功时返回浮点数, 否则返回原始数据
    """
    try:
        return float(value)
    except (ValueError, TypeError):
        return value


def safe_convert_to_int(
    value: Any,
) -> int | Any:
    """尝试将值转换为整数

    Args:
        value (Any): 用于转换的值
    Returns:
        (int | Any): 转换成功时返回整数, 否则返回原始数据
    """
    try:
        return int(value)
    except (ValueError, TypeError):
        return value


def open_file_as_bytes(
    input_file: Path,
) -> bytes:
    """读取文件并且以二进制模式打开

    Args:
        input_file (Path): 要打开的文件
    Returns:
        bytes: 二进制文件
    Raises:
        FileNotFoundError: 当文件不存在时抛出
        PermissionError: 当权限不足无法读取文件时抛出
        IsADirectoryError: 当路径是目录而不是文件时抛出
        OSError: 当操作系统错误导致无法读取文件时抛出
        Exception: 当发生其他未知错误时抛出
    """
    try:
        with input_file.open("rb") as binary_file:
            return binary_file.read()
    except FileNotFoundError as e:
        logger.error("文件未找到: '%s'", input_file)
        raise e
    except PermissionError as e:
        logger.error("权限不足, 无法读取文件: '%s'", input_file)
        raise e
    except IsADirectoryError as e:
        logger.error("路径是一个目录, 不是文件: '%s'", input_file)
        raise e
    except OSError as e:
        logger.error("操作系统错误, 无法读取文件: '%s', 错误信息: %s", input_file, e)
        raise e
    except Exception as e:
        logger.error("读取文件时发生未知错误: '%s'", input_file)
        raise e


def save_bytes_to_file(
    bytes_file: bytes,
    output_path: Path,
) -> None:
    """将二进制文件保存为文件

    Args:
        bytes_file (bytes): 二进制文件
        output_path (Path): 保存二进制文件的路径
    Raises:
        FileNotFoundError: 当输出目录不存在时抛出
        PermissionError: 当权限不足无法写入文件时抛出
        IsADirectoryError: 当输出路径是目录而不是文件时抛出
        OSError: 当操作系统错误导致无法写入文件时抛出
        Exception: 当发生其他未知错误时抛出
    """
    try:
        with output_path.open("wb") as f:
            f.write(bytes_file)
    except FileNotFoundError as e:
        logger.error("输出目录不存在: '%s'", output_path)
        raise e
    except PermissionError as e:
        logger.error("权限不足, 无法读取文件: '%s'", output_path)
        raise e
    except IsADirectoryError as e:
        logger.error("输出路径是一个目录, 不是文件: '%s'", output_path)
        raise e
    except OSError as e:
        logger.error("操作系统错误, 无法写入文件: '%s', 错误信息: %s", output_path, e)
        raise e
    except Exception as e:
        logger.error("写入输出文件时发生未知错误: '%s'", output_path)
        raise e


def is_utf8_bom_encoding_file(
    file_path: Path,
) -> bool:
    """检测文本文件是否为 UTF8 BOM 编码保存

    Args:
        file_path (Path): 文本文件路径

    Returns:
        bool: 检测结果
    """
    with open(file_path, "rb") as file:
        bom = file.read(3)
        return bom == b"\xef\xbb\xbf"


def detect_encoding(
    file_path: Path,
) -> str:
    """
    检测文件的编码格式

    Args:
        file_path (Path): 文件路径

    Returns:
        str: 检测到的编码格式, 如 'utf-8' 或 'gbk'
    """
    if is_utf8_bom_encoding_file(file_path):
        logger.debug("'%s' 文件编码格式为 UTF8 BOM", file_path)
        return "utf-8-sig"

    with open(file_path, "rb") as f:
        raw_data = f.read()

    try:
        raw_data.decode("utf-8")
        logger.debug("'%s' 文件编码格式为 UTF8", file_path)
        return "utf-8"
    except UnicodeDecodeError:
        logger.debug("'%s' 文件编码格式为 GBK", file_path)
        return "gbk"


def lowercase_dict_keys(
    d: dict[str, Any],
) -> dict[str, Any]:
    """
    递归地将字典中的键转换为小写形式, 但如果转换后会导致键重复, 则保留原键名不变. 同时处理嵌套的字典

    Args:
        d (dict[str, Any]): 输入的字典

    Returns:
        (dict[str, Any]): 处理后的新字典
    """
    if not isinstance(d, dict):
        return d

    # 用于存储转换后的键值对
    new_dict = {}
    # 用于记录准备转换的键，以检查转换后是否会产生冲突
    lower_key_mapping = {}

    # 首先收集所有键及其转换为小写后的形式
    for key, value in d.items():
        lower_key = key.lower() if isinstance(key, str) else key
        if lower_key in lower_key_mapping:
            # 如果小写后的键已经存在, 说明会产生冲突, 将这两个键都保留原样
            lower_key_mapping[lower_key] = None  # 标记为冲突
        else:
            lower_key_mapping[lower_key] = key  # 存储原始键

    # 构建新字典
    for key, value in d.items():
        # 检查这个键的小写形式是否与其他键冲突
        lower_key = key.lower() if isinstance(key, str) else key

        # 如果键是字符串且小写形式映射到多个原始键 (即存在冲突), 则保留原键
        if isinstance(key, str) and lower_key_mapping.get(lower_key) is None:
            final_key = key  # 保留原始键
        # 如果小写键只对应一个原始键, 则使用小写键
        elif isinstance(key, str) and lower_key_mapping.get(lower_key) == key:
            final_key = lower_key
        else:
            final_key = key  # 非字符串键保持不变

        # 递归处理值
        if isinstance(value, dict):
            new_dict[final_key] = lowercase_dict_keys(value)
        elif isinstance(value, list):
            # 如果值是列表, 检查列表中的每个元素是否为字典
            new_dict[final_key] = [lowercase_dict_keys(item) if isinstance(item, dict) else item for item in value]
        else:
            new_dict[final_key] = value

    return new_dict


def extend_list_to_length(
    lst: list[Any],
    target_length: int,
    fill_value: str | None = "",
) -> list[Any]:
    """将列表扩展到指定长度

    Args:
        lst (list[Any]): 原始列表
        target_length (int): 扩充到的指定长度
        fill_value (str | None): 填充的内容
    Returns:
        list[str]: 扩展长度后的列表
    """
    if len(lst) < target_length:
        lst.extend([fill_value] * (target_length - len(lst)))
    return lst


def is_admin_on_windows() -> bool:
    """检测当前进程是否以管理员权限运行

    Returns:
        bool: 当使用管理员运行时返回 True
    """
    try:
        return ctypes.windll.shell32.IsUserAnAdmin()
    except AttributeError:
        return False


def is_root_on_linux() -> bool:
    """检测当前进程是否以管理员权限运行

    Returns:
        bool: 当使用管理员运行时返回 True
    """
    return getpass.getuser() == "root"


def unload_specific_module(
    module_name: str,
) -> None:
    """从内存卸载指定的 Python 模块

    Args:
        module_name (str): 要卸载的模块名称
    """
    if module_name not in sys.modules:
        logger.info("'%s' 模块未被加载", module_name)
        return

    logger.info("卸载 '%s' 模块中", module_name)
    # 尝试调用模块的清理函数
    if hasattr(module_name, "cleanup") and callable(getattr(module_name, "cleanup")):
        try:
            getattr(module_name, "cleanup")()
        except Exception:  # pylint: disable=broad-exception-caught
            pass

        # 如果模块有句柄属性
        if hasattr(module_name, "_handle"):
            handle = module_name._handle  # pylint: disable=protected-access
            if sys.platform.startswith("win"):
                ctypes.windll.kernel32.FreeLibrary(handle)
            elif sys.platform.startswith("linux"):
                ctypes.CDLL(None).dlclose(handle)

    # 删除 sys.modules 中的条目
    del sys.modules[module_name]
    logger.info("卸载模块: '%s'", module_name)

    # 如果有子模块, 也要删除
    module_keys_to_remove = []
    for key in sys.modules:
        if key.startswith(module_name + "."):
            module_keys_to_remove.append(key)

    for key in module_keys_to_remove:
        del sys.modules[key]
        logger.info("卸载 '%s' 子模块: '%s'", module_name, key)

    # 从局部命名空间删除模块引用 (如果在全局命名空间中)
    if module_name in globals():
        del globals()[module_name]

    gc.collect()
    logger.info("模块 '%s' 已彻底卸载", module_name)


def generate_random_string(
    length: int | None = 8,
    chars: str | None = None,
    include_uppercase: bool | None = True,
    include_lowercase: bool | None = True,
    include_digits: bool | None = True,
    include_special: bool | None = False,
) -> str:
    """
    生成随机字符串

    Args:
        length (int | None): 字符串长度, 默认为 8
        chars (str | None): 自定义字符集，如果提供则忽略其他参数
        include_uppercase (bool | None): 是否包含大写字母
        include_lowercase (bool | None): 是否包含小写字母
        include_digits (bool | None): 是否包含数字
        include_special (bool | None): 是否包含特殊字符

    Returns:
        str: 生成的随机字符串
    """
    if chars is not None:
        char_pool = chars
    else:
        char_pool = ""
        if include_uppercase:
            char_pool += string.ascii_uppercase
        if include_lowercase:
            char_pool += string.ascii_lowercase
        if include_digits:
            char_pool += string.digits
        if include_special:
            char_pool += "!@#$%^&*"

    if not char_pool:
        raise ValueError("字符池不能为空")

    return "".join(random.choice(char_pool) for _ in range(length))


def normalized_filepath(
    filepath: str | Path,
) -> Path:
    """将输入的路径转换为绝对路径

    Args:
        filepath (str | Path): 原始的路径
    Returns:
        Path: 绝对路径
    """
    if filepath is not None:
        filepath = Path(filepath).absolute()

    logger.debug("解析成绝对路径后的路径: '%s'", filepath)
    return filepath


def is_http_or_https(url: str) -> bool:
    """检测字符串是否为 HTTP / HTTPS 链接

    Args:
        url (str): 待检测的字符串

    Returns:
        bool: 如果是 HTTPS / HTTP 链接返回 True
    """
    try:
        result = urlparse(url)
        # result.scheme 获取协议部分 (如 http, https)
        # result.netloc 确保有域名部分 (避免 "https://" 这种空链接)
        return result.scheme in ("http", "https") and bool(result.netloc)
    except Exception:
        return False



"""配置管理"""

import os
import logging
import sys
from pathlib import Path

LOGGER_NAME = os.getenv("ANI2XCUR_LOGGER_NAME", "Ani2xcur")
"""日志器名字"""

LOGGER_LEVEL = int(os.getenv("ANI2XCUR_LOGGER_LEVEL", str(logging.INFO)))
"""日志等级"""

LOGGER_COLOR = os.getenv("ANI2XCUR_LOGGER_COLOR") not in ["0", "False", "false", "None", "none", "null"]
"""日志颜色"""

ROOT_PATH = Path(__file__).parent
"""Ani2xcur 根目录"""

LINUX_CURSOR_SOURCE_PATH = ROOT_PATH / "source"
"""Linux 鼠标指针补全文件目录"""

IMAGE_MAGICK_WINDOWS_DOWNLOAD_URL = os.getenv(
    "IMAGE_MAGICK_WINDOWS_DOWNLOAD_URL", "https://modelscope.cn/models/licyks/invokeai-core-model/resolve/master/pypatchmatch/ImageMagick-7.1.2-Q16-HDRI.zip"
)
"""下载 ImageMagick Windows 版的 URL"""

IMAGE_MAGICK_WINDOWS_INSTALL_PATH = Path(os.getenv("ProgramFiles", r"C:\Program Files")) / "ImageMagick-7.1.2-Q16-HDRI"
"""Windows 默认 ImageMagick 安装路径"""

ANI2XCUR_REPOSITORY_URL = os.getenv("ANI2XCUR_REPOSITORY_URL", "https://github.com/licyk/ani2xcur-cli")
"""Ani2xcur 仓库地址"""

WIN2XCUR_REPOSITORY_URL = os.getenv("WIN2XCUR_REPOSITORY_URL", "https://github.com/quantum5/win2xcur")
"""Win2xcur 仓库地址"""

ANI2XCUR_CONFIG_PATH = (Path("~").expanduser() / "AppData" / "Roaming" / "ani2xcur") if sys.platform == "win32" else (Path("~").expanduser() / ".config" / "ani2xcur")
"""Ani2xcur 配置文件路径"""

SMART_FINDER_SEARCH_DEPTH = int(os.getenv("ANI2XCUR_SMART_FINDER_SEARCH_DEPTH", "3"))
"""Ani2xcur 智能搜索鼠标指针配置文件的深度"""



"""GTK 环境配置工具"""

import re
import configparser
from pathlib import Path

GTK4_CONFIG_PATH = Path("~/.config/gtk-4.0/settings.ini").expanduser()
"""GTK 4.0 配置文件路径"""

GTK3_CONFIG_PATH = Path("~/.config/gtk-3.0/settings.ini").expanduser()
"""GTK 3.0 配置文件路径"""

GTK2_CONFIG_PATH = Path("~/.gtkrc-2.0").expanduser()
"""GTK 2.0 配置文件路径"""


def read_gtk2_config(
    config_path: Path,
) -> dict[str, str]:
    """读取 GTK 2.0 配置文件并返回配置字典

    Args:
        config_path (Path): GTK 2.0 配置文件路径
    Returns:
        (dict[str, str]): GTK 2.0 配置字典
    """
    config: dict[str, str] = {}

    with open(config_path, "r", encoding="utf-8") as file:
        for line in file:
            line = line.strip()
            # 跳过注释行和空行
            if line.startswith("#") or not line:
                continue

            # 匹配配置项格式: key=value
            match = re.match(r"^(\w+(?:-\w+)*)=(.*)$", line)
            if match:
                key, value = match.groups()
                # 移除值两端的引号 (如果有的话)
                if value.startswith('"') and value.endswith('"'):
                    value = value[1:-1]
                elif value.startswith("'") and value.endswith("'"):
                    value = value[1:-1]

                config[key] = value

    return config


def write_gtk2_config(
    config_path: Path,
    updates: dict[str, str],
) -> None:
    """修改 GTK 2.0 配置文件

    Args:
        config_path (Path): 配置文件路径
        updates (dict[str, str]): 要更新的内容字典
    """
    lines = []
    found_keys = set()

    if config_path.exists():
        with open(config_path, "r", encoding="utf-8") as f:
            lines = f.readlines()

    new_lines = []
    for line in lines:
        stripped = line.strip()
        # 匹配当前的 key
        match = re.match(r"^([\w-]+)=", stripped)
        if match:
            key = match.group(1)
            if key in updates:
                # 针对字符串值，通常 GTK2 推荐使用双引号
                val = updates[key]
                # 如果是数字 (如 size), 直接写; 如果是字符串 (如 theme), 加引号
                # 这里简单处理: 如果输入是纯数字且 key 包含 size, 则不加引号
                if "size" in key or val.isdigit():
                    new_lines.append(f"{key}={val}\n")
                else:
                    new_lines.append(f'{key}="{val}"\n')
                found_keys.add(key)
                continue
        new_lines.append(line)

    # 追加文件中不存在的新配置项
    for key, val in updates.items():
        if key not in found_keys:
            if "size" in key or str(val).isdigit():
                new_lines.append(f"{key}={val}\n")
            else:
                new_lines.append(f'{key}="{val}"\n')

    with open(config_path, "w", encoding="utf-8") as f:
        f.writelines(new_lines)


def get_gtk4_cursor_theme() -> str | None:
    """获取 GTK4 标准方案的当前使用的鼠标指针配置名称

    Returns:
        (str | None): 当前使用的鼠标指针名称
    """
    config = configparser.ConfigParser()
    config.read(GTK4_CONFIG_PATH)
    if "Settings" in config and "gtk-cursor-theme-name" in config["Settings"]:
        return config.get("Settings", "gtk-cursor-theme-name")
    return None


def get_gtk3_cursor_theme() -> str | None:
    """获取 GTK3 标准方案的当前使用的鼠标指针配置名称

    Returns:
        (str | None): 当前使用的鼠标指针名称
    """
    config = configparser.ConfigParser()
    config.read(GTK3_CONFIG_PATH)
    if "Settings" in config and "gtk-cursor-theme-name" in config["Settings"]:
        return config.get("Settings", "gtk-cursor-theme-name")
    return None


def get_gtk2_cursor_theme() -> str | None:
    """获取 GTK2 标准方案的当前使用的鼠标指针配置名称

    Returns:
        (str | None): 当前使用的鼠标指针名称
    """
    if not GTK2_CONFIG_PATH.is_file():
        return None

    config = read_gtk2_config(GTK2_CONFIG_PATH)
    return config.get("gtk-cursor-theme-name")


def get_gtk4_cursor_size() -> int | None:
    """获取 GTK4 标准方案的当前使用的鼠标指针大小

    Returns:
        (int | None): 当前使用的鼠标指针大小
    """
    config = configparser.ConfigParser()
    config.read(GTK4_CONFIG_PATH)
    if "Settings" in config and "gtk-cursor-theme-size" in config["Settings"]:
        return int(config.get("Settings", "gtk-cursor-theme-size"))
    return None


def get_gtk3_cursor_size() -> int | None:
    """获取 GTK3 标准方案的当前使用的鼠标指针大小

    Returns:
        (int | None): 当前使用的鼠标指针大小
    """
    config = configparser.ConfigParser()
    config.read(GTK3_CONFIG_PATH)
    if "Settings" in config and "gtk-cursor-theme-size" in config["Settings"]:
        return int(config.get("Settings", "gtk-cursor-theme-size"))
    return None


def get_gtk2_cursor_size() -> int | None:
    """获取 GTK2 标准方案的当前使用的鼠标指针大小

    Returns:
        (int | None): 当前使用的鼠标指针大小
    """
    if not GTK2_CONFIG_PATH.is_file():
        return None

    config = read_gtk2_config(GTK2_CONFIG_PATH)
    return int(config.get("gtk-cursor-theme-size"))


def set_gtk4_cursor_theme(
    cursor_name: str,
) -> None:
    """设置 GTK4 标准方案的当前使用的鼠标指针配置名称

    Args:
        cursor_name (str): 要设置的鼠标指针配置名称
    """
    GTK4_CONFIG_PATH.parent.mkdir(parents=True, exist_ok=True)
    config = configparser.ConfigParser()
    config.read(GTK4_CONFIG_PATH)
    if "Settings" not in config:
        config["Settings"] = {}

    config["Settings"]["gtk-cursor-theme-name"] = cursor_name
    with open(GTK4_CONFIG_PATH, "w", encoding="utf-8") as f:
        config.write(f, space_around_delimiters=False)


def set_gtk3_cursor_theme(
    cursor_name: str,
) -> None:
    """设置 GTK3 标准方案的当前使用的鼠标指针配置名称

    Args:
        cursor_name (str): 要设置的鼠标指针配置名称
    """
    GTK3_CONFIG_PATH.parent.mkdir(parents=True, exist_ok=True)
    config = configparser.ConfigParser()
    config.read(GTK3_CONFIG_PATH)
    if "Settings" not in config:
        config["Settings"] = {}

    config["Settings"]["gtk-cursor-theme-name"] = cursor_name
    with open(GTK3_CONFIG_PATH, "w", encoding="utf-8") as f:
        config.write(f, space_around_delimiters=False)


def set_gtk2_cursor_theme(
    cursor_name: str,
) -> None:
    """设置 GTK2 标准方案的当前使用的鼠标指针配置名称

    Args:
        cursor_name (str): 要设置的鼠标指针配置名称
    """
    GTK2_CONFIG_PATH.parent.mkdir(parents=True, exist_ok=True)
    write_gtk2_config(config_path=GTK2_CONFIG_PATH, updates={"gtk-cursor-theme-name": cursor_name})


def set_gtk4_cursor_size(
    cursor_size: int,
) -> None:
    """设置 GTK4 标准方案的当前使用的鼠标指针大小

    Args:
        cursor_size (int): 要设置的鼠标指针大小
    """
    GTK4_CONFIG_PATH.parent.mkdir(parents=True, exist_ok=True)
    config = configparser.ConfigParser()
    config.read(GTK4_CONFIG_PATH)
    if "Settings" not in config:
        config["Settings"] = {}

    config["Settings"]["gtk-cursor-theme-size"] = str(cursor_size)
    with open(GTK4_CONFIG_PATH, "w", encoding="utf-8") as f:
        config.write(f, space_around_delimiters=False)


def set_gtk3_cursor_size(
    cursor_size: int,
) -> None:
    """设置 GTK3 标准方案的当前使用的鼠标指针大小

    Args:
        cursor_size (str): 要设置的鼠标指针大小
    """
    GTK3_CONFIG_PATH.parent.mkdir(parents=True, exist_ok=True)
    config = configparser.ConfigParser()
    config.read(GTK3_CONFIG_PATH)
    if "Settings" not in config:
        config["Settings"] = {}

    config["Settings"]["gtk-cursor-theme-size"] = str(cursor_size)
    with open(GTK3_CONFIG_PATH, "w", encoding="utf-8") as f:
        config.write(f, space_around_delimiters=False)


def set_gtk2_cursor_size(
    cursor_size: int,
) -> None:
    """设置 GTK2 标准方案的当前使用的鼠标指针大小

    Args:
        cursor_size (str): 要设置的鼠标指针大小
    """
    GTK2_CONFIG_PATH.parent.mkdir(parents=True, exist_ok=True)
    write_gtk2_config(config_path=GTK2_CONFIG_PATH, updates={"gtk-cursor-theme-size": str(cursor_size)})



"""压缩 / 解压工具"""

import os
import zipfile
import tarfile
import lzma
from pathlib import Path
from typing import Iterable

import rarfile
import zstandard as zstd
import py7zr

from ani2xcur.config import (
    LOGGER_NAME,
    LOGGER_LEVEL,
    LOGGER_COLOR,
)
from ani2xcur.logger import get_logger

logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


SUPPORTED_ARCHIVE_FORMAT = [
    ".zip",
    ".7z",
    ".rar",
    ".tar",
    ".tar.Z",
    ".tar.lz",
    ".tar.lzma",
    ".tar.bz2",
    ".tar.7z",
    ".tar.gz",
    ".tar.xz",
    ".tar.zst",
]
"""支持的压缩包格式列表"""


def is_supported_archive_format(
    archive_path: Path,
) -> bool:
    """查看压缩包是否支持解压或是否支持压缩成的格式

    Args:
        archive_path (Path): 压缩包路径
    Returns:
        bool: 支持结果
    """
    name = archive_path.name
    for f in SUPPORTED_ARCHIVE_FORMAT:
        if name.endswith(f):
            return True
    return False


def extract_archive(
    archive_path: Path,
    extract_to: Path,
) -> None:
    """解压支持的压缩包

    Args:
        archive_path (Path): 压缩包路径
        extract_to (Path): 解压到的路径
    Raises:
        ValueError: 不支持解压时
    """
    if not is_supported_archive_format(archive_path):
        raise ValueError(f"不支持的压缩格式: {archive_path}")

    name = archive_path.name.lower()
    extract_to.mkdir(parents=True, exist_ok=True)

    logger.info("将 '%s' 解压到 '%s' 中", archive_path, extract_to)

    if name.endswith(".zip"):
        with zipfile.ZipFile(archive_path, "r") as zip_ref:
            zip_ref.extractall(extract_to)

    if name.endswith(".tar"):
        with tarfile.open(archive_path, "r") as tar_ref:
            tar_ref.extractall(extract_to)
        return

    if name.endswith(".tar.gz"):
        with tarfile.open(archive_path, "r:gz") as tar_ref:
            tar_ref.extractall(extract_to)
            return

    if name.endswith(".tar.bz2"):
        with tarfile.open(archive_path, "r:bz2") as tar_ref:
            tar_ref.extractall(extract_to)
        return

    if name.endswith(".tar.xz"):
        with tarfile.open(archive_path, "r:xz") as tar_ref:
            tar_ref.extractall(extract_to)
        return

    if name.endswith(".tar.lzma") or name.endswith(".tlz"):
        with lzma.open(archive_path, "rb") as f:
            with tarfile.open(fileobj=f) as tar_ref:
                tar_ref.extractall(extract_to)
        return

    if name.endswith(".tar.zst") or name.endswith(".tar..zst"):
        with open(archive_path, "rb") as fh:
            dctx = zstd.ZstdDecompressor()
            with dctx.stream_reader(fh) as reader:
                with tarfile.open(fileobj=reader) as tar_ref:
                    tar_ref.extractall(extract_to)
        return

    if name.endswith(".7z"):
        with py7zr.SevenZipFile(archive_path, mode="r") as archive:
            archive.extractall(path=extract_to)
        return

    if name.endswith(".rar"):
        with rarfile.RarFile(archive_path, mode="r") as archive:
            archive.extractall(path=extract_to)
        return


def create_archive(
    sources: Iterable[Path],
    archive_path: Path,
) -> None:
    """根据扩展名创建压缩包

    Args:
        sources (Iterable[Path]): 要打包的文件或目录的列表
        archive_path (Path): 压缩文件保存路径
    Raises:
        ValueError: 不支持的压缩或不能写入的格式
    """

    def _add_to_tar(
        tar_ref: tarfile.TarFile,
        src: Path,
        arcname: str | None = None,
    ) -> None:
        if arcname is None:
            arcname = src.name
        tar_ref.add(str(src), arcname=arcname)

    if not is_supported_archive_format(archive_path):
        raise ValueError(f"不支持的压缩格式: {archive_path}")

    name = archive_path.name.lower()
    sources = list(sources)
    archive_path.parent.mkdir(parents=True, exist_ok=True)

    logger.info("将 '%s' 压缩并保存到 '%s' 中", sources, archive_path)

    # zip
    if name.endswith(".zip"):
        with zipfile.ZipFile(archive_path, "w", compression=zipfile.ZIP_DEFLATED) as zf:
            for src in sources:
                if src.is_dir():
                    for root, _, files in os.walk(src):
                        root_p = Path(root)
                        for f in files:
                            fp = root_p / f
                            arcname = str(fp.relative_to(src.parent))
                            zf.write(str(fp), arcname=arcname)
                else:
                    arcname = src.name
                    zf.write(str(src), arcname=arcname)
        return

    # 7z
    if name.endswith(".7z"):
        with py7zr.SevenZipFile(archive_path, mode="w") as archive:
            for src in sources:
                if src.is_dir():
                    archive.writeall(str(src), arcname=src.name)
                else:
                    archive.write(str(src), arcname=src.name)
        return

    # rar 写入不支持
    if name.endswith(".rar"):
        raise ValueError("创建 .rar 压缩包不受支持")

    # 直接由 tarfile 支持的压缩模式
    if name.endswith(".tar"):
        mode = "w"
    elif name.endswith(".tar.gz") or name.endswith(".tgz"):
        mode = "w:gz"
    elif name.endswith(".tar.bz2") or name.endswith(".tbz2"):
        mode = "w:bz2"
    elif name.endswith(".tar.xz") or name.endswith(".txz"):
        mode = "w:xz"
    else:
        mode = None

    if mode is not None:
        with tarfile.open(archive_path, mode) as tar_ref:
            for src in sources:
                _add_to_tar(tar_ref, src)
        return

    # .tar.lzma 直接用 lzma 包装 tar 的文件对象
    if name.endswith(".tar.lzma") or name.endswith(".tlz"):
        with lzma.open(archive_path, "wb") as f:
            with tarfile.open(fileobj=f, mode="w") as tar_ref:
                for src in sources:
                    _add_to_tar(tar_ref, src)
        return

    # .tar.zst 使用 zstandard 的流写入
    if name.endswith(".tar.zst") or name.endswith(".tar..zst"):
        with open(archive_path, "wb") as fh:
            cctx = zstd.ZstdCompressor()
            with cctx.stream_writer(fh) as compressor:
                with tarfile.open(fileobj=compressor, mode="w") as tar_ref:
                    for src in sources:
                        _add_to_tar(tar_ref, src)
        return



"""LXQT 桌面环境配置工具"""

import configparser
from pathlib import Path

LXQT_CONFIG_PATH = Path("~/.config/lxqt/session.conf").expanduser()
"""LXQT 桌面的配置文件路径"""


def get_lxqt_cursor_theme() -> str | None:
    """获取 LXQT 桌面当前使用的鼠标指针配置名称

    Returns:
        (str | None): 当前使用的鼠标指针名称
    """
    config = configparser.ConfigParser()
    config.read(LXQT_CONFIG_PATH)
    if "General" in config and "cursor_theme" in config["General"]:
        return config.get("General", "cursor_theme")
    return None


def get_lxqt_cursor_size() -> int | None:
    """获取 LXQT 桌面当前使用的鼠标指针大小

    Returns:
        (int | None): 当前使用的鼠标指针大小
    """
    config = configparser.ConfigParser()
    config.read(LXQT_CONFIG_PATH)
    if "General" in config and "cursor_size" in config["General"]:
        return int(config.get("General", "cursor_size"))
    return None


def set_lxqt_cursor_theme(
    cursor_name: str,
) -> None:
    """设置 LXQT 桌面当前使用的鼠标指针配置名称

    Args:
        cursor_name (str): 要设置的鼠标指针配置名称
    """
    LXQT_CONFIG_PATH.parent.mkdir(parents=True, exist_ok=True)
    config = configparser.ConfigParser()
    config.read(LXQT_CONFIG_PATH)
    if "General" not in config:
        config["General"] = {}

    config["General"]["cursor_theme"] = cursor_name
    with open(LXQT_CONFIG_PATH, "w", encoding="utf-8") as f:
        config.write(f, space_around_delimiters=False)


def set_lxqt_cursor_size(
    cursor_size: int,
) -> None:
    """设置 LXQT 桌面当前使用的鼠标指针大小

    Args:
        cursor_size (int): 要设置的鼠标指针大小
    """
    LXQT_CONFIG_PATH.parent.mkdir(parents=True, exist_ok=True)
    config = configparser.ConfigParser()
    config.read(LXQT_CONFIG_PATH)
    if "General" not in config:
        config["General"] = {}

    config["General"]["cursor_size"] = str(cursor_size)
    with open(LXQT_CONFIG_PATH, "w", encoding="utf-8") as f:
        config.write(f, space_around_delimiters=False)



"""KDE 桌面环境配置工具"""

import shutil

from ani2xcur.cmd import run_cmd
from ani2xcur.utils import safe_convert_to_int


def get_kde_cursor_theme() -> str | None:
    """获取 KDE 桌面当前使用的鼠标指针配置名称

    Returns:
        (str | None): 当前使用的鼠标指针名称
    """
    if shutil.which("kreadconfig5"):
        executable = "kreadconfig5"
    elif shutil.which("kreadconfig6"):
        executable = "kreadconfig6"
    else:
        return None

    result = run_cmd(
        [
            executable,
            "--file",
            "kcminputrc",
            "--group",
            "Mouse",
            "--key",
            "cursorTheme",
        ],
        live=False,
        check=False,
    )

    if isinstance(result, str):
        result = result.strip()

    if result == "":
        result = None

    return result


def get_kde_cursor_size() -> int | None:
    """获取 KDE 桌面当前使用的鼠标指针大小

    Returns:
        (int | None): 当前使用的鼠标指针大小
    """
    if shutil.which("kreadconfig5"):
        executable = "kreadconfig5"
    elif shutil.which("kreadconfig6"):
        executable = "kreadconfig6"
    else:
        return None

    result = run_cmd(
        [
            executable,
            "--file",
            "kcminputrc",
            "--group",
            "Mouse",
            "--key",
            "cursorSize",
        ],
        live=False,
        check=False,
    )

    if isinstance(result, str):
        result = result.strip()

    if result == "":
        result = None

    return safe_convert_to_int(result)


def set_kde_cursor_theme(
    cursor_name: str,
) -> None:
    """设置 KDE 桌面当前使用的鼠标指针配置名称

    Args:
        cursor_name (str): 要设置的鼠标指针配置名称
    """
    if shutil.which("kwriteconfig5"):
        executable = "kwriteconfig5"
    elif shutil.which("kwriteconfig6"):
        executable = "kwriteconfig6"
    else:
        return

    run_cmd(
        [executable, "--file", "kcminputrc", "--group", "Mouse", "--key", "cursorTheme", cursor_name],
        live=False,
        check=False,
    )


def set_kde_cursor_size(
    cursor_size: int,
) -> None:
    """设置 KDE 桌面当前使用的鼠标指针大小

    Args:
        cursor_size (int): 要设置的鼠标指针大小
    """
    if shutil.which("kwriteconfig5"):
        executable = "kwriteconfig5"
    elif shutil.which("kwriteconfig6"):
        executable = "kwriteconfig6"
    else:
        return

    run_cmd(
        [executable, "--file", "kcminputrc", "--group", "Mouse", "--key", "cursorSize", str(cursor_size)],
        live=False,
        check=False,
    )



"""Mate 桌面环境配置工具"""

import shutil

from ani2xcur.cmd import run_cmd
from ani2xcur.utils import safe_convert_to_int


def get_mate_cursor_theme() -> str | None:
    """获取 Mate 桌面当前使用的鼠标指针配置名称

    Returns:
        (str | None): 当前使用的鼠标指针名称
    """
    if not shutil.which("gsettings"):
        return None

    result = run_cmd(
        ["gsettings", "get", "org.mate.peripherals-mouse", "cursor-theme"],
        live=False,
        check=False,
    )

    if isinstance(result, str):
        result = result.strip()

    if result == "":
        result = None

    return result


def get_mate_cursor_size() -> int | None:
    """获取 Mate 桌面当前使用的鼠标指针大小

    Returns:
        (int | None): 当前使用的鼠标指针大小
    """
    if not shutil.which("gsettings"):
        return None

    result = run_cmd(
        ["gsettings", "get", "org.mate.peripherals-mouse", "cursor-size"],
        live=False,
        check=False,
    )
    if isinstance(result, str):
        result = result.strip()

    if result == "":
        result = None

    return safe_convert_to_int(result)


def set_mate_cursor_theme(
    cursor_name: str,
) -> None:
    """设置 Mate 桌面当前使用的鼠标指针配置名称

    Args:
        cursor_name (str): 要设置的鼠标指针配置名称
    """
    if not shutil.which("gsettings"):
        return

    run_cmd(
        ["gsettings", "set", "org.mate.peripherals-mouse", "cursor-theme", cursor_name],
        live=False,
        check=False,
    )


def set_mate_cursor_size(
    cursor_size: int,
) -> None:
    """设置 Mate 桌面当前使用的鼠标指针大小

    Args:
        cursor_size (int): 要设置的鼠标指针大小
    """
    if not shutil.which("gsettings"):
        return

    run_cmd(
        ["gsettings", "get", "org.mate.peripherals-mouse", "cursor-size", str(cursor_size)],
        live=False,
        check=False,
    )



"""鼠标指针缩放工具"""

from win2xcur.cursor import CursorFrame


def apply_to_frames(
    frames: list[CursorFrame],
    *,
    scale: float,
) -> None:
    """将缩放应用到鼠标指针文件的帧上

    Args:
        frames (list[CursorFrame]): 鼠标指针的帧列表
        scale (float): 缩放倍数
    """
    for frame in frames:
        for cursor in frame:
            cursor.image.scale(
                int(round(cursor.image.width * scale)),
                int(round(cursor.image.height) * scale),
            )



"""Linux 鼠标指针配置文件解析"""

from pathlib import Path
from typing import (
    Any,
    Literal,
    Protocol,
    TypedDict,
    cast,
)

from ani2xcur.config_parse.parse import (
    ParsedINF,
    parse_inf_file,
)


CursorShemeDesktopEntry = TypedDict(
    "CursorShemeDesktopEntry",
    {
        "Icon Theme": dict[str, str | list[str]],  # 鼠标指针信息
    },
    total=False,
)
"""预处理后的鼠标指针配置 DesktopEntry 结构"""
DesktopEntrySectionName = Literal[
    "Icon Theme",  # 鼠标指针信息
]
"""DesktopEntry 文件已知键名"""


class DesktopEntrySectionDict(Protocol):
    """DesktopEntry 选项结构字典"""

    def get(  # pylint: disable=missing-function-docstring
        self,
        key: Literal["var", "constant"],
        default: Any = ...,
    ) -> str | dict[str, str | list[str]] | Any: ...


class KnownDesktopEntrySections(Protocol):
    """已知的 DesktopEntry 结构"""

    def __getitem__(
        self,
        key: DesktopEntrySectionName,
    ) -> DesktopEntrySectionDict: ...
    def __contains__(
        self,
        key: object,
    ) -> bool: ...


def preprocess_desktop_entry_to_cursor_scheme(
    parsed: ParsedINF,
) -> CursorShemeDesktopEntry:
    """将 ParsedDesktopEntry 处理为鼠标指针已知键的扁平结构

    返回包含常见节的字典:
    - Icon Theme: 该节的 var & constant 鼠标指针信息 (若存在)

    Args:
        parsed (ParsedINF): DesktopEntry 类型字典
    Returns:
        CursorShemeDesktopEntry: 预处理后的鼠标指针 DesktopEntry 信息字典
    Raises:
        ValueError: 当鼠标指针配置文件不完整时
    """

    out: CursorShemeDesktopEntry = {}
    parsed_known = cast(KnownDesktopEntrySections, parsed)

    if "Icon Theme" in parsed_known:
        # 合并 var 和 constant（如果需要可扩展）
        out["Icon Theme"] = parsed_known["Icon Theme"].get("var", {})
    else:
        raise ValueError("未找到 Icon Theme 键, 鼠标指针配置不完整")

    return out


def parse_desktop_entry_content(
    desktop_entry_path: Path,
) -> CursorShemeDesktopEntry:
    """从 DesktopEntry 文件中获取鼠标指针配置数据

    Args:
        desktop_entry_path (Path): 鼠标指针的 DesktopEntry 配置文件
    Returns:
        CursorShemeDesktopEntry: 鼠标指针配置数据
    Raises:
        ValueError: 当鼠标指针配置文件不完整时
    """
    desktop_entry = parse_inf_file(desktop_entry_path)
    try:
        return preprocess_desktop_entry_to_cursor_scheme(desktop_entry)
    except ValueError as e:
        raise e


def dict_to_desktop_entry_strings_format(
    data_dict: dict[str, str],
) -> str:
    """
    将字典转换为 Desktop Entry 文件 [<Var>] 部分的格式

    Args:
        data_dict (dict[str, str]): 包含键值对的字典

    Returns:
        str: 格式化的字符串
    """
    lines = []
    for key, value in data_dict.items():
        # 使用制表符或空格对齐，保持与原格式一致
        line = f"{key}={value}"
        lines.append(line)

    return "\n".join(lines)



"""Linux 鼠标指针管理工具"""

import stat
from typing import TypedDict
from pathlib import Path

from ani2xcur.manager.base import (
    CurrentCursorInfo,
    CurrentCursorInfoList,
    CursorMap,
    CURSOR_KEYS,
    LocalCursor,
)
from ani2xcur.config_parse.linux import parse_desktop_entry_content
from ani2xcur.file_operations.file_manager import (
    copy_files,
    get_file_list,
    remove_files,
)
from ani2xcur.manager.base import (
    CursorSchemesList,
    LINUX_ICONS_PATH,
    LINUX_USER_ICONS_PATH,
)
from ani2xcur.manager.desktop_config.base import check_linux_cursor_size_value
from ani2xcur.manager.desktop_config.cinnamon import (
    set_cinnamon_cursor_size,
    set_cinnamon_cursor_theme,
    get_cinnamon_cursor_size,
    get_cinnamon_cursor_theme,
)
from ani2xcur.manager.desktop_config.gnome import (
    set_gnome_cursor_size,
    set_gnome_cursor_theme,
    get_gnome_cursor_size,
    get_gnome_cursor_theme,
)
from ani2xcur.manager.desktop_config.gtk import (
    set_gtk2_cursor_size,
    set_gtk2_cursor_theme,
    set_gtk3_cursor_size,
    set_gtk3_cursor_theme,
    set_gtk4_cursor_size,
    set_gtk4_cursor_theme,
    get_gtk2_cursor_size,
    get_gtk2_cursor_theme,
    get_gtk3_cursor_size,
    get_gtk3_cursor_theme,
    get_gtk4_cursor_size,
    get_gtk4_cursor_theme,
)
from ani2xcur.manager.desktop_config.kde import (
    set_kde_cursor_size,
    set_kde_cursor_theme,
    get_kde_cursor_size,
    get_kde_cursor_theme,
)
from ani2xcur.manager.desktop_config.lxqt import (
    set_lxqt_cursor_size,
    set_lxqt_cursor_theme,
    get_lxqt_cursor_size,
    get_lxqt_cursor_theme,
)
from ani2xcur.manager.desktop_config.mate import (
    set_mate_cursor_size,
    set_mate_cursor_theme,
    get_mate_cursor_size,
    get_mate_cursor_theme,
)
from ani2xcur.manager.desktop_config.x_org import (
    set_x_resources_cursor_size,
    set_x_resources_cursor_theme,
    get_x_resources_cursor_size,
    get_x_resources_cursor_theme,
)
from ani2xcur.manager.desktop_config.xdg import (
    set_xdg_cursor_theme,
    get_xdg_cursor_theme,
)
from ani2xcur.manager.desktop_config.xfce import (
    set_xfce_cursor_size,
    set_xfce_cursor_theme,
    get_xfce_cursor_size,
    get_xfce_cursor_theme,
)
from ani2xcur.manager.desktop_config.xsettings import (
    set_gtk_xsettings_cursor_size,
    set_gtk_xsettings_cursor_theme,
    get_gtk_xsettings_cursor_size,
    get_gtk_xsettings_cursor_theme,
)
from ani2xcur.config import (
    LOGGER_LEVEL,
    LOGGER_COLOR,
    LOGGER_NAME,
)
from ani2xcur.logger import get_logger

logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


class InstallLinuxSchemeInfo(TypedDict):
    """Linux 鼠标指针安装信息"""

    scheme_name: str
    """鼠标指针名称"""

    cursor_paths: list[Path]
    """鼠标指针文件列表"""

    vars_dict: dict[str, str]
    """INF 文件中的变量表"""

    cursor_map: CursorMap
    """鼠标指针类型与对应的路径地图"""


def extract_scheme_info_from_desktop_entry(
    desktop_entry_file: Path,
) -> InstallLinuxSchemeInfo:
    """从 Desktop Entry 文件中获取鼠标指针配置

    Args:
        desktop_entry_file (Path): Desktop Entry 文件路径
    Returns:
        InstallLinuxSchemeInfo: 鼠标指针安装配置
    Raises:
        FileNotFoundError: 鼠标指针文件缺失时
    """
    scheme_info: InstallLinuxSchemeInfo = {}
    desktop_entry_content = parse_desktop_entry_content(desktop_entry_file)
    scheme_name = desktop_entry_content["Icon Theme"]["Name"]
    cursor_path = desktop_entry_file.parent / "cursors"
    if not cursor_path.is_dir():
        raise FileNotFoundError(f"未找到 {cursor_path} 目录, 无法搜索已有的鼠标指针文件")

    cursor_paths = get_file_list(
        path=desktop_entry_file.parent / "cursors",
        max_depth=0,
    )
    cursor_key_paths = {x.name: x for x in cursor_paths}
    cursor_map: CursorMap = {}
    vars_dict = desktop_entry_content["Icon Theme"]
    for win, linux in zip(CURSOR_KEYS["win"], CURSOR_KEYS["linux"]):
        if linux in cursor_key_paths:
            src = dst = cursor_key_paths[linux]
        else:
            src = dst = None
        cursor_map[win] = {
            "src_path": src,
            "dst_path": dst,
        }

    scheme_info["scheme_name"] = scheme_name
    scheme_info["cursor_paths"] = cursor_paths
    scheme_info["vars_dict"] = vars_dict
    scheme_info["cursor_map"] = cursor_map
    return scheme_info


def list_linux_cursors() -> CursorSchemesList:
    """列出 Linux 系统中已有的鼠标指针

    Returns:
        CursorSchemesList: 本地已安装的鼠标指针列表
    """
    cursors_list: CursorSchemesList = []
    icon_user_paths = get_file_list(LINUX_USER_ICONS_PATH, max_depth=0, include_dirs=True)
    icon_system_paths = get_file_list(LINUX_ICONS_PATH, max_depth=0, include_dirs=True)
    logger.debug("用户图标目录文件列表: %s", icon_user_paths)
    logger.debug("系统图标目录文件列表: %s", icon_system_paths)
    icon_paths = icon_user_paths + icon_system_paths
    for path in icon_paths:
        cursors_dir = path / "cursors"
        if not cursors_dir.is_dir():
            logger.debug("'%s' 缺少鼠标指针文件夹", path)
            continue

        logger.debug("获取 '%s' 鼠标指针的文件列表", path)
        cursor_files = get_file_list(cursors_dir)
        cursors: LocalCursor = {}
        cursors["name"] = path.name
        cursors["cursor_files"] = cursor_files
        cursors["install_paths"] = [path]
        cursors_list.append(cursors)

    return cursors_list


def set_linux_cursor_theme(
    cursor_name: str,
) -> None:
    """设置 Linux 桌面当前使用的鼠标指针配置名称

    Args:
        cursor_name (str): 要设置的鼠标指针配置名称
    Raises:
        ValueError: 鼠标指针不存在时
    """
    cursors = [x["name"] for x in list_linux_cursors()]
    if cursor_name not in cursors:
        logger.error("鼠标指针 '%s' 不存在", cursor_name)
        raise ValueError(f"鼠标指针 {cursor_name} 不存在")

    logger.info("将 Linux 系统中使用的鼠标指针主题设置为 '%s'", cursor_name)
    set_cinnamon_cursor_theme(cursor_name)
    set_gnome_cursor_theme(cursor_name)
    set_gtk2_cursor_theme(cursor_name)
    set_gtk3_cursor_theme(cursor_name)
    set_gtk4_cursor_theme(cursor_name)
    set_kde_cursor_theme(cursor_name)
    set_lxqt_cursor_theme(cursor_name)
    set_mate_cursor_theme(cursor_name)
    set_x_resources_cursor_theme(cursor_name)
    set_xdg_cursor_theme(cursor_name)
    set_xfce_cursor_theme(cursor_name)
    set_gtk_xsettings_cursor_theme(cursor_name)
    logger.info("Linux 鼠标指针主题已设置为 '%s'", cursor_name)


def set_linux_cursor_size(
    cursor_size: int,
) -> None:
    """设置 Linux 桌面当前使用的鼠标指针大小

    Args:
        cursor_size (int): 要设置的鼠标指针大小
    Raises:
        TypeError: 鼠标指针大小的值不是整数时
        ValueError: 鼠标指针大小的值超过合法范围时
    """
    try:
        check_linux_cursor_size_value(cursor_size)
    except TypeError as e:
        raise TypeError("鼠标指针大小的值必须为整数") from e
    except ValueError as e:
        raise ValueError("鼠标指针大小的值超过合法范围") from e

    logger.info("将 Linux 系统中使用的鼠标指针大小设置为 %s", cursor_size)
    set_cinnamon_cursor_size(cursor_size)
    set_gnome_cursor_size(cursor_size)
    set_gtk2_cursor_size(cursor_size)
    set_gtk3_cursor_size(cursor_size)
    set_gtk4_cursor_size(cursor_size)
    set_kde_cursor_size(cursor_size)
    set_lxqt_cursor_size(cursor_size)
    set_mate_cursor_size(cursor_size)
    set_x_resources_cursor_size(cursor_size)
    set_xfce_cursor_size(cursor_size)
    set_gtk_xsettings_cursor_size(cursor_size)
    logger.info("鼠标指针大小已设置为 %s", cursor_size)


def get_linux_cursor_info() -> CurrentCursorInfoList:
    """获取 Linux 当前鼠标指针信息

    Returns:
        CurrentCursorInfoList: 桌面平台的当前鼠标指针信息列表
    """
    logger.info("获取 Linux 系统的鼠标指针状态")
    info_list: CurrentCursorInfoList = []

    # Cinnamon
    cinnamon: CurrentCursorInfo = {}
    cinnamon["platform"] = "Cinnamon"
    cinnamon["cursor_name"] = get_cinnamon_cursor_theme()
    cinnamon["cursor_size"] = get_cinnamon_cursor_size()
    info_list.append(cinnamon)

    # Gnome
    gnome: CurrentCursorInfo = {}
    gnome["platform"] = "Gnome"
    gnome["cursor_name"] = get_gnome_cursor_theme()
    gnome["cursor_size"] = get_gnome_cursor_size()
    info_list.append(gnome)

    # GTK 2.0
    gtk2: CurrentCursorInfo = {}
    gtk2["platform"] = "GTK 2.0"
    gtk2["cursor_name"] = get_gtk2_cursor_theme()
    gtk2["cursor_size"] = get_gtk2_cursor_size()
    info_list.append(gtk2)

    # GTK 3.0
    gtk3: CurrentCursorInfo = {}
    gtk3["platform"] = "GTK 3.0"
    gtk3["cursor_name"] = get_gtk3_cursor_theme()
    gtk3["cursor_size"] = get_gtk3_cursor_size()
    info_list.append(gtk3)

    # GTK 4.0
    gtk4: CurrentCursorInfo = {}
    gtk4["platform"] = "GTK 4.0"
    gtk4["cursor_name"] = get_gtk4_cursor_theme()
    gtk4["cursor_size"] = get_gtk4_cursor_size()
    info_list.append(gtk4)

    # KDE
    kde: CurrentCursorInfo = {}
    kde["platform"] = "KDE"
    kde["cursor_name"] = get_kde_cursor_theme()
    kde["cursor_size"] = get_kde_cursor_size()
    info_list.append(kde)

    # LXQT
    lxqt: CurrentCursorInfo = {}
    lxqt["platform"] = "LXQT"
    lxqt["cursor_name"] = get_lxqt_cursor_theme()
    lxqt["cursor_size"] = get_lxqt_cursor_size()
    info_list.append(lxqt)

    # Mate
    mate: CurrentCursorInfo = {}
    mate["platform"] = "Mate"
    mate["cursor_name"] = get_mate_cursor_theme()
    mate["cursor_size"] = get_mate_cursor_size()
    info_list.append(mate)

    # X.Org
    x_org: CurrentCursorInfo = {}
    x_org["platform"] = "X.Org"
    x_org["cursor_name"] = get_x_resources_cursor_theme()
    x_org["cursor_size"] = get_x_resources_cursor_size()
    info_list.append(x_org)

    # XDG
    xdg: CurrentCursorInfo = {}
    xdg["platform"] = "XDG"
    xdg["cursor_name"] = ",".join([str(x) for x in get_xdg_cursor_theme()])
    xdg["cursor_size"] = None
    info_list.append(xdg)

    # Xfce
    xfce: CurrentCursorInfo = {}
    xfce["platform"] = "Xfce"
    xfce["cursor_name"] = get_xfce_cursor_theme()
    xfce["cursor_size"] = get_xfce_cursor_size()
    info_list.append(xfce)

    # X Settings
    xfce: CurrentCursorInfo = {}
    xfce["platform"] = "X Settings"
    xfce["cursor_name"] = get_gtk_xsettings_cursor_theme()
    xfce["cursor_size"] = get_gtk_xsettings_cursor_size()
    info_list.append(xfce)

    return info_list


def delete_linux_cursor(
    cursor_name: str,
) -> None:
    """删除 Linux 系统上指定的鼠标指针

    Args:
        cursor_name (str): 要删除的鼠标指针名称
    Raises:
        RuntimeError: 删除鼠标指针文件失败时
        ValueError: 指定的鼠标指针不存在时
    """
    cursors = list_linux_cursors()
    if cursor_name not in [x["name"] for x in cursors]:
        raise ValueError(f"鼠标指针 {cursor_name} 不存在")

    logger.info("从 Linux 系统删除 '%s' 鼠标指针中", cursor_name)
    for scheme in cursors:
        if cursor_name == scheme["name"]:
            # 清理鼠标指针文件
            for file in scheme["cursor_files"]:
                if not file.exists():
                    logger.debug("鼠标指针文件 '%s' 不存在", file)
                    continue

                try:
                    logger.debug("清理鼠标指针文件 '%s'", file)
                    remove_files(file)
                except OSError as e:
                    logger.error("删除 '%s' 鼠标指针所使用的指针文件 '%s' 发生错误: %s\n可尝试使用管理员权限运行 Ani2xcur 进行删除, 或者尝试手动删除文件")
                    raise RuntimeError(f"删除 {cursor_name} 鼠标指针所使用的指针文件 {file} 发生错误: {e}\n可尝试使用管理员权限运行 Ani2xcur 进行删除, 或者尝试手动删除文件") from e

            # 清理鼠标指针的父文件夹
            for file in scheme["install_paths"]:
                if not file.is_dir():
                    logger.debug("鼠标指针文件的父文件夹 '%s' 不存在", file)
                    continue

                try:
                    logger.debug("清理鼠标指针文件的父文件夹 '%s'", file)
                    remove_files(file)
                except OSError as e:
                    logger.error("清理 '%s' 鼠标指针文件的残留文件夹 '%s' 发生错误: %s\n可尝试使用管理员权限运行 Ani2xcur 进行删除, 或者尝试手动删除文件", cursor_name, file, e)
                    raise RuntimeError(f"清理 {cursor_name} 鼠标指针文件的残留文件夹 {file} 发生错误: {e}\n可尝试使用管理员权限运行 Ani2xcur 进行删除, 或者尝试手动删除文件") from e

    logger.info("从 Linux 系统删除 '%s' 鼠标指针完成", cursor_name)


def install_linux_cursor(
    desktop_entry_file: Path,
    cursor_install_path: Path | None = None,
) -> None:
    """通过 DesktopEntry 配置文件安装鼠标指针

    Args:
        desktop_entry_file (Path): 鼠标指针配置文件路径
        cursor_install_path (Path | None): 自定义鼠标指针文件安装路径, 当为 None 时使用默认安装路径
    Raises:
        FileNotFoundError: 鼠标指针中缺少 cursors 文件夹时
        RuntimeError:  复制鼠标指针文件夹发生失败时
    """
    cursors_path = desktop_entry_file.parent / "cursors"
    if not cursors_path.is_dir():
        raise FileNotFoundError(f"在 {cursors_path} 中缺少 cursors 文件夹, 鼠标指针已损坏")

    scheme_info = extract_scheme_info_from_desktop_entry(desktop_entry_file)
    logger.debug("解析到的 DesktopEntry 配置: %s", scheme_info)
    cursor_name = scheme_info["scheme_name"]
    src = desktop_entry_file.parent
    if cursor_install_path is not None:
        dst = cursor_install_path / cursor_name
    else:
        dst = LINUX_USER_ICONS_PATH / cursor_name

    logger.info("将 '%s' 鼠标指针安装到 '%s' 中", cursor_name, dst)

    try:
        copy_files(src, dst)
    except OSError as e:
        logger.error("复制鼠标指针 '%s' 到 '%s' 时发生错误: %s\n可尝试使用 root 权限运行 Ani2xcur 进行鼠标指针安装操作", src, dst, e)
        raise RuntimeError(f"复制鼠标指针 {src} 到 {dst} 时发生错误: {e}\n可尝试使用 root 权限运行 Ani2xcur 进行鼠标指针安装操作") from e

    logger.info("'%s' 鼠标指针已安装到 '%s'", cursor_name, dst)


def export_linux_cursor(
    cursor_name: str,
    output_path: Path,
    custom_install_path: Path | None = None,
) -> Path:
    """将系统中指定的鼠标指针方案导出为文件

    Args:
        cursor_name (str): 要导出的鼠标指针方案的名称
        output_path (Path): 鼠标指针导出的路径
        custom_install_path (Path | None): 自定义鼠标指针安装时的文件安装路径
    Returns:
        Path: 鼠标指针导出的文件路径
    Raises:
        ValueError: 鼠标指针在当前环境中不存在时
        RuntimeError: 导出鼠标指针文件发生失败时
    """
    cursors = list_linux_cursors()
    cursor_data = None
    for data in cursors:
        if data["name"] == cursor_name:
            cursor_data = data

    if cursor_data is None:
        raise ValueError(f"鼠标指针 {cursor_name} 不存在")

    src = cursor_data["install_paths"][0]
    save_dir = output_path / cursor_name

    logger.info("将 '%s' 鼠标指针导出到 '%s' 中", cursor_name, save_dir)

    try:
        copy_files(src, save_dir)
    except OSError as e:
        logger.error("从 '%s' 导出鼠标指针文件到 '%s' 时发生错误: %s", src, save_dir, e)
        raise RuntimeError(f"从 {src} 导出鼠标指针文件到 {save_dir} 时发生错误: {e}") from e

    generate_install_script(
        cursor_name=cursor_name,
        save_dir=save_dir,
        custom_install_path=custom_install_path,
    )

    logger.info("'%s' 鼠标指针导出到 '%s' 完成", cursor_name, save_dir)
    return save_dir


def generate_install_script(
    cursor_name: str,
    save_dir: Path,
    custom_install_path: Path | None = None,
) -> None:
    """生成 Linux 鼠标指针安装脚本

    Args:
        cursor_name (str): 鼠标指针名称
        save_dir (Path): 安装脚本保存路径
        custom_install_path (Path | None): 自定义鼠标指针安装文件
    """
    install_path = str(custom_install_path) if custom_install_path is not None else "${HOME}/.icons"
    sh_content = r"""
#!/bin/bash

cursor_path=$(cd "$(dirname "$0")" ; pwd)

install_cursor() {
    mkdir -p "{{__INSTALL_PATH__}}" || return 1
    cp -r "${cursor_path}" "{{__INSTALL_PATH__}}" || return 1
    return 0
}

get_cursor_set_command() {
    local cursor_name=$1
    cat<<EOF
Cinnamon:
    gsettings set org.cinnamon.desktop.interface cursor-theme "${cursor_name}"

Gnome:
    gsettings set org.gnome.desktop.interface cursor-theme "${cursor_name}"

Mate:
    gsettings set org.mate.peripherals-mouse cursor-theme "${cursor_name}"

KDE:
    kwriteconfig5 --file kcminputrc --group Mouse --key cursorTheme "${cursor_name}"

Xfce:
    xfconf-query --channel xsettings --property /Gtk/CursorThemeName --set "${cursor_name}"
EOF
}

echo "将鼠标指针安装至 '{{__INSTALL_PATH__}}/{{__CURSOR_NAME__}}'"

if install_cursor; then
    echo "'{{__CURSOR_NAME__}}' 鼠标指针安装完成, 可使用运行下面的命令启用该鼠标指针"
    get_cursor_set_command "{{__CURSOR_NAME__}}"
else
    echo "鼠标指针安装到 '{{__INSTALL_PATH__}}/{{__CURSOR_NAME__}}' 失败, 请检查是否有 root 权限或者检查目录是否存在"
    exit 1
fi
""".strip()
    sh_content = sh_content.replace("{{__INSTALL_PATH__}}", install_path).replace("{{__CURSOR_NAME__}}", cursor_name)

    sh_path = save_dir / "install_cursor.sh"
    with open(sh_path, "w", encoding="utf-8") as f:
        f.write(sh_content)

    current_permissions = sh_path.stat().st_mode
    sh_path.chmod(current_permissions | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)



"""Windows 鼠标指针管理工具"""

from typing import TypedDict
from pathlib import Path

from ani2xcur.config_parse.win import parse_inf_file_content
from ani2xcur.logger import get_logger
from ani2xcur.config import (
    LOGGER_LEVEL,
    LOGGER_COLOR,
    LOGGER_NAME,
)
from ani2xcur.file_operations.file_manager import (
    remove_files,
    copy_files,
)
from ani2xcur.config_parse.win import dict_to_inf_strings_format
from ani2xcur.manager.base import (
    CURSOR_KEYS,
    CurrentCursorInfo,
    CurrentCursorInfoList,
    CursorMap,
    LocalCursor,
    CursorSchemesList,
)
from ani2xcur.manager.desktop_config.base import check_windows_cursor_size_value
from ani2xcur.manager.desktop_config.windows import (
    WINDOWS_CURSOR_CURSORS_SCHEME_PATH,
    get_windows_cursor_size,
    get_windows_cursor_theme,
    has_var_string,
)
from ani2xcur.manager.desktop_config.windows import (
    set_windows_cursor_theme as apply_windows_cursor_theme,
)
from ani2xcur.manager.desktop_config.windows import (
    set_windows_cursor_size as apply_windows_cursor_size,
)
from ani2xcur.manager.regedit import (
    RegistryAccess,
    RegistryRootKey,
    RegistryValueType,
    registry_enum_values,
    registry_delete_value,
    registry_set_value,
    registry_query_value,
)
from ani2xcur.utils import extend_list_to_length
from ani2xcur.manager.desktop_config.windows import expand_var_string

logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


class InstallWindowsSchemeInfo(TypedDict):
    """Windows 鼠标指针安装信息"""

    scheme_name: str
    """鼠标指针名称"""

    cursor_paths: list[Path]
    """鼠标指针文件列表"""

    default_reg: str
    """默认安装到注册表的鼠标指针方案, 使用 [Scheme.Reg] 中的值"""

    default_dst_cursor_paths: list[Path]
    """默认复制到的目的路径的鼠标指针文件列表"""

    vars_dict: dict[str, str]
    """INF 文件中的变量表"""

    cursor_map: CursorMap
    """鼠标指针类型与对应的路径地图"""


def extract_scheme_info_from_inf(
    inf_file: Path,
) -> InstallWindowsSchemeInfo:
    """从 INF 文件中获取鼠标指针配置

    Args:
        inf_file (Path): INF 文件路径
    Returns:
        InstallWindowsSchemeInfo: 鼠标指针安装配置
    Raises:
        ValueError: 鼠标指针配置文件中的注册表信息不合法时
    """

    def _get_real_path(
        x: str,
    ) -> Path:
        return Path(expand_var_string(x.replace('"', "").replace("'", ""), vars_dict))

    scheme_info: InstallWindowsSchemeInfo = {}
    cursor_map: CursorMap = {}
    inf_file_content = parse_inf_file_content(inf_file)
    scheme_reg = parse_scheme_reg_string(inf_file_content["Scheme.Reg"][0])
    vars_dict = inf_file_content["Strings"]
    scheme_name = vars_dict["SCHEME_NAME"]
    cursor_files = inf_file_content["Scheme.Cur"]

    # 检查 [Scheme.Reg] 长度合法性
    if len(scheme_reg) != 5:
        raise ValueError(f"鼠标指针配置中的注册表配置不合法, 配置长度: {len(scheme_reg)}")

    # 将鼠标指针字段扩展到合适长度
    cursor_reg_paths: list[str] = extend_list_to_length(scheme_reg[4].split(","), target_length=len(CURSOR_KEYS["win"]))

    # 检查 [Scheme.Reg] 中鼠标指针数量
    if len(cursor_reg_paths) > len(CURSOR_KEYS["win"]):
        raise ValueError(f"鼠标指针配置中指定的鼠标指针数量不合法, 指定的鼠标指针数量: {len(cursor_reg_paths)}")

    # 重新生成 [Scheme.Reg] 字段
    default_reg = generate_scheme_reg_string(
        key=scheme_reg[0],
        sub_key=scheme_reg[1],
        value_name=scheme_reg[2],
        dtype=scheme_reg[3],
        value=",".join(cursor_reg_paths),
    )

    # 统一路径分隔符 (Linux 中无法处理正确 `\` 路径分隔符字符串)
    cursor_reg_paths = [x.replace(r"\\", "/").replace("\\", "/") for x in cursor_reg_paths]

    # 获取鼠标指针配置中指针文件的实际安装路径列表
    default_dst_cursor_paths = [_get_real_path(x) for x in cursor_reg_paths if _get_real_path(x).is_file()]

    # 记录鼠标指针原文件路径和安装到的实际路径, 并记录成字典
    for key, value in zip(CURSOR_KEYS["win"], cursor_reg_paths):
        if value.strip() != "":
            dst_path = _get_real_path(value)
            src_path = inf_file.parent / dst_path.name
            if not src_path.is_file():
                src_path = None
        else:
            dst_path = None
            src_path = None

        cursor_map[key] = {
            "dst_path": dst_path,
            "src_path": src_path,
        }

    # 鼠标指针原文件列表
    cursor_paths = [inf_file.parent / x for x in cursor_files if (inf_file.parent / x).is_file()]

    # 生成字典
    scheme_info["scheme_name"] = scheme_name
    scheme_info["cursor_paths"] = cursor_paths
    scheme_info["default_reg"] = default_reg
    scheme_info["vars_dict"] = vars_dict
    scheme_info["default_dst_cursor_paths"] = default_dst_cursor_paths
    scheme_info["cursor_map"] = cursor_map
    return scheme_info


def parse_scheme_reg_string(
    scheme_reg_string: str,
) -> list[str]:
    """解析 INF 字符串中 [Scheme.Reg] 格式的字符串, 将字符串分割成列表

    Args:
        scheme_reg_string (str): 需要解析的字符串

    Returns:
        list[str]: 解析后的字符串列表
    """
    result: list[str] = []
    current = ""
    in_single_quotes = False
    in_double_quotes = False
    i = 0

    while i < len(scheme_reg_string):
        char = scheme_reg_string[i]

        if char == '"' and not in_single_quotes:
            # 切换双引号状态 (仅当不在单引号内)
            in_double_quotes = not in_double_quotes
            i += 1
        elif char == "'" and not in_double_quotes:
            # 切换单引号状态 (仅当不在双引号内)
            in_single_quotes = not in_single_quotes
            i += 1
        elif char == "," and not in_single_quotes and not in_double_quotes:
            # 只有在非引号状态下遇到逗号才分割
            result.append(current)
            current = ""
            i += 1
        else:
            # 添加字符到当前项
            current += char
            i += 1

    # 添加最后一项
    if current or scheme_reg_string.endswith(","):
        result.append(current)

    # 去除每项首尾的引号 (如果存在)
    cleaned_result: list[str] = []
    for item in result:
        cleaned_item = item
        # 如果同时以单引号开头结尾, 则去除单引号
        if len(cleaned_item) >= 2 and cleaned_item.startswith("'") and cleaned_item.endswith("'"):
            cleaned_item = cleaned_item[1:-1]
        # 如果同时以双引号开头结尾, 则去除双引号
        elif len(cleaned_item) >= 2 and cleaned_item.startswith('"') and cleaned_item.endswith('"'):
            cleaned_item = cleaned_item[1:-1]
        cleaned_result.append(cleaned_item)

    return cleaned_result


def generate_scheme_reg_string(
    key: str,
    sub_key: str,
    value_name: str,
    dtype: str,
    value: str,
) -> str:
    """生成 INF 字符串中 [Scheme.Reg] 格式的字符串

    [Scheme.Reg] 用于创建 Windows 注册表值, 格式: `<注册表根键>,<注册表子路径>,<键名>,<数据类型>,<值>`

    Args:
        key (str): 注册表根键
        sub_key (str): 注册表子路径
        value_name (str): 键名
        dtype (str): 数据类型
        value (str): 值
    """
    return f'{key},"{sub_key}","{value_name}",{dtype},"{value}"'


def list_windows_cursors() -> CursorSchemesList:
    """列出 Windows 系统中已有的鼠标指针

    Returns:
        CursorSchemesList: 本地已安装的鼠标指针列表
    """
    schemes = registry_enum_values(
        sub_key=WINDOWS_CURSOR_CURSORS_SCHEME_PATH,
        key=RegistryRootKey.CURRENT_USER,
        access=RegistryAccess.READ,
    )
    cursors_list: CursorSchemesList = []
    for name, data in schemes.items():
        cursor_files = [Path(expand_var_string(x)) for x in data.split(",") if x.strip() != "" and Path(expand_var_string(x)).is_file()]
        install_paths = list({x.parent for x in cursor_files})
        cursors: LocalCursor = {}
        cursors["name"] = name
        cursors["cursor_files"] = cursor_files
        cursors["install_paths"] = install_paths
        cursors_list.append(cursors)

    return cursors_list


def set_windows_cursor_theme(
    cursor_name: str,
) -> None:
    """设置 Windows 桌面当前使用的鼠标指针配置名称

    Args:
        cursor_name (str): 要设置的鼠标指针配置名称
    Raises:
        ValueError: 鼠标指针不存在时
    """
    cursors = [x["name"] for x in list_windows_cursors()]
    if cursor_name not in cursors:
        logger.error("鼠标指针 '%s 不存在", cursor_name)
        raise ValueError(f"鼠标指针 {cursor_name} 不存在")

    logger.info("将 Windows 系统中使用的鼠标指针主题设置为 '%s'", cursor_name)
    apply_windows_cursor_theme(cursor_name)
    logger.info("Windows 鼠标指针主题已设置为 '%s'", cursor_name)


def set_windows_cursor_size(
    cursor_size: int,
) -> None:
    """设置 Windows 桌面当前使用的鼠标指针大小

    Args:
        cursor_size (int): 要设置的鼠标指针大小
    Raises:
        TypeError: 鼠标指针大小的值不是整数时
        ValueError: 鼠标指针大小的值超过合法范围时
    """
    try:
        check_windows_cursor_size_value(cursor_size)
    except TypeError as e:
        raise TypeError("鼠标指针大小的值必须为整数") from e
    except ValueError as e:
        raise ValueError("鼠标指针大小的值超过合法范围") from e
    logger.info("将 Windows 系统中使用的鼠标指针大小设置为 %s", cursor_size)
    apply_windows_cursor_size(cursor_size)
    logger.info("鼠标指针大小已设置为 %s", cursor_size)


def get_windows_cursor_info() -> CurrentCursorInfoList:
    """获取 Windows 当前鼠标指针信息

    Returns:
        CurrentCursorInfoList: 桌面平台的当前鼠标指针信息列表
    """
    logger.info("获取 Windows 系统的鼠标指针状态")
    info: CurrentCursorInfo = {}
    info_list: CurrentCursorInfoList = []
    info["platform"] = "Windows"
    info["cursor_name"] = get_windows_cursor_theme()
    info["cursor_size"] = get_windows_cursor_size()
    info_list.append(info)
    return info_list


def delete_windows_cursor(
    cursor_name: str,
) -> None:
    """删除 Windows 系统上指定的鼠标指针

    Args:
        cursor_name (str): 要删除的鼠标指针名称
    Raises:
        RuntimeError: 删除鼠标指针文件失败时
        ValueError: 指定的鼠标指针不存在或者正在被使用时
    """
    cursors = list_windows_cursors()
    if cursor_name not in [x["name"] for x in cursors]:
        raise ValueError(f"鼠标指针 {cursor_name} 不存在")

    if cursor_name == get_windows_cursor_theme():
        raise ValueError(f"鼠标指针 {cursor_name} 正在被使用, 无法删除")

    logger.info("从 Windows 系统删除 '%s' 鼠标指针中", cursor_name)
    for scheme in cursors:
        if cursor_name == scheme["name"]:
            # 清理鼠标指针文件
            for file in scheme["cursor_files"]:
                if not file.exists():
                    logger.debug("鼠标指针文件 '%s' 不存在", file)
                    continue

                try:
                    logger.debug("清理鼠标指针文件 '%s'", file)
                    remove_files(file)
                except OSError as e:
                    logger.error(
                        "删除 '%s' 鼠标指针所使用的指针文件 '%s' 发生错误: %s\n可尝试使用管理员权限运行 Ani2xcur 进行删除, 或者尝试手动删除文件",
                        cursor_name,
                        file,
                        e,
                    )
                    raise RuntimeError(f"删除 {cursor_name} 鼠标指针所使用的指针文件 {file} 发生错误: {e}\n可尝试使用管理员权限运行 Ani2xcur 进行删除, 或者尝试手动删除文件") from e

            # 清理鼠标指针的父文件夹
            for file in scheme["install_paths"]:
                if not file.is_dir():
                    logger.debug("鼠标指针文件的父文件夹 '%s' 不存在", file)
                    continue

                if any(file.iterdir()):
                    logger.debug("鼠标指针文件的父文件夹 '%s' 不为空, 跳过清理", file)
                    continue

                try:
                    logger.debug("清理鼠标指针文件的父文件夹 '%s'", file)
                    remove_files(file)
                except OSError as e:
                    logger.error(
                        "清理 '%s' 鼠标指针文件的残留文件夹 '%s' 发生错误: %s\n可尝试使用管理员权限运行 Ani2xcur 进行删除, 或者尝试手动删除文件",
                        cursor_name,
                        file,
                        e,
                    )
                    raise RuntimeError(f"清理 {cursor_name} 鼠标指针文件的残留文件夹 {file} 发生错误: {e}\n可尝试使用管理员权限运行 Ani2xcur 进行删除, 或者尝试手动删除文件") from e

    # 清理注册表中对应的鼠标指针方案
    registry_delete_value(
        name=cursor_name,
        sub_key=WINDOWS_CURSOR_CURSORS_SCHEME_PATH,
        key=RegistryRootKey.CURRENT_USER,
        access=RegistryAccess.SET_VALUE,
    )

    logger.info("从 Windows 系统删除 '%s' 鼠标指针完成", cursor_name)


def install_windows_cursor(
    inf_file: Path,
    cursor_install_path: Path | None = None,
) -> None:
    """通过 INF 配置文件安装鼠标指针

    Args:
        inf_file (Path): 鼠标指针配置文件路径
        cursor_install_path (Path | None): 自定义鼠标指针文件安装路径, 当为 None 时使用 INF 配置文件中的路径
    Returns:
        PermissionError: 当复制鼠标指针时没有权限进行复制时
    """
    scheme_info = extract_scheme_info_from_inf(inf_file)
    copy_paths: list[tuple[Path, Path]] = []
    cursor_paths_in_reg: list[str] = []
    cursor_name = scheme_info["scheme_name"]
    install_path = None

    # 生成复制路径列表
    if cursor_install_path is not None:
        # 使用自定义安装路径
        install_path = cursor_install_path
        for _, cursor_pair in scheme_info["cursor_map"].items():
            src = cursor_pair["src_path"]
            if src is not None:
                dst = cursor_install_path / cursor_name / cursor_pair["dst_path"].name
                cursor_paths_in_reg.append(str(dst))
                copy_paths.append((src, dst))
            else:
                cursor_paths_in_reg.append("")

        # 生成需要写入注册表的方案对应值
        reg_scheme_value = ",".join(cursor_paths_in_reg)
    else:
        for _, cursor_pair in scheme_info["cursor_map"].items():
            src = cursor_pair["src_path"]
            if src is not None:
                install_path = cursor_pair["dst_path"].parent
                dst = cursor_pair["dst_path"]
                copy_paths.append((src, dst))

        # 生成需要写入注册表的方案对应值, 使用原始值
        reg_scheme_value = parse_scheme_reg_string(scheme_info["default_reg"])[4]

    logger.info("将 '%s' 鼠标指针安装到 '%s' 中", cursor_name, install_path)

    # 复制鼠标指针文件
    for src, dst in copy_paths:
        try:
            dst.parent.mkdir(parents=True, exist_ok=True)
            copy_files(src, dst)
        except OSError as e:
            logger.error(
                "复制 '%s' 到 '%s' 时发送错误: %s\n可尝试使用管理员权限运行 Ani2xcur",
                src,
                dst,
                e,
            )
            raise PermissionError(f"复制 {src} 到 {dst} 时发送错误: {e}\n可尝试使用管理员权限运行 Ani2xcur, 再执行鼠标指针安装操作") from e

    # 将方案写入注册表
    reg_type = RegistryValueType.EXPAND_SZ if has_var_string(reg_scheme_value) else RegistryValueType.SZ
    registry_set_value(
        name=cursor_name,
        value=reg_scheme_value,
        reg_type=reg_type,
        sub_key=WINDOWS_CURSOR_CURSORS_SCHEME_PATH,
        key=RegistryRootKey.CURRENT_USER,
        access=RegistryAccess.SET_VALUE,
    )
    logger.info("'%s' 鼠标指针已安装到 Windows 系统中", cursor_name)


def export_windows_cursor(
    cursor_name: str,
    output_path: Path,
    custom_install_path: Path | None = None,
) -> Path:
    """将系统中指定的鼠标指针方案导出为文件

    Args:
        cursor_name (str): 要导出的鼠标指针方案的名称
        output_path (Path): 鼠标指针导出的路径
        custom_install_path (Path | None): 自定义鼠标指针安装时的文件安装路径
    Returns:
        Path: 鼠标指针导出的文件路径
    Raises:
        ValueError: 鼠标指针在当前环境中不存在时
    """
    cursors = list_windows_cursors()
    cursor_data = None
    for data in cursors:
        if data["name"] == cursor_name:
            cursor_data = data

    if cursor_data is None:
        raise ValueError(f"鼠标指针 {cursor_name} 不存在")

    config_dict = generate_cursor_scheme_config(
        cursor_name=cursor_name,
        custom_install_path=custom_install_path,
    )

    reg_content = generate_cursor_scheme_inf_string(
        destination_dirs=config_dict["destination_dirs"],
        wreg=config_dict["wreg"],
        scheme_reg=config_dict["scheme_reg"],
        scheme_cur=config_dict["scheme_cur"],
        strings=config_dict["strings"],
    )

    save_dir = output_path / cursor_name
    save_dir.mkdir(parents=True, exist_ok=True)
    inf_file_path = save_dir / "AutoSetup.inf"
    logger.info("将 '%s' 鼠标指针导出到 '%s' 中", cursor_name, save_dir)
    for cursor in config_dict["cursor_src_file"]:
        copy_files(cursor, save_dir)

    # Windows 系统只能使用 GBK 编码保存 INF
    with open(inf_file_path, "w", encoding="gbk") as file:
        file.write(reg_content)

    logger.info("'%s' 鼠标指针导出到 '%s' 完成", cursor_name, save_dir)
    return save_dir


def generate_cursor_scheme_inf_string(
    destination_dirs: str,
    wreg: str,
    scheme_reg: str,
    scheme_cur: str,
    strings: str,
) -> str:
    """生成鼠标指针安装配置文件

    Args:
        destination_dirs (str): [DestinationDirs] 字段
        wreg (str): [Wreg] 字段
        scheme_reg (str): [Scheme.Reg] 字段
        scheme_cur (str): [Scheme.Cur] 字段
        strings (str): [Strings] 字段
    Returns:
        str: 鼠标指针的 INF 字符串
    """
    inf_content = r"""
[Version]
signature="$CHICAGO$"


[DefaultInstall]
CopyFiles = Scheme.Cur
AddReg    = Scheme.Reg,Wreg


[DestinationDirs]
Scheme.Cur = {{DESTINATION_DIRS}}


[Scheme.Reg]
{{SCHEME_REG}}


[Wreg]
{{WREG}}


[Scheme.Cur]
{{SCHEME_CUR}}


[Strings]
{{STRING_VARS}}

""".strip()

    return (
        inf_content.replace(r"{{DESTINATION_DIRS}}", destination_dirs.strip())
        .replace(r"{{WREG}}", wreg.strip())
        .replace(r"{{SCHEME_REG}}", scheme_reg.strip())
        .replace(r"{{SCHEME_CUR}}", scheme_cur.strip())
        .replace(r"{{STRING_VARS}}", strings.strip())
    )


def generate_cursor_scheme_config(
    cursor_name: str,
    custom_install_path: Path | None = None,
) -> dict[str, str]:
    """生成鼠标指针的配置, 配置字典字段:
    - `cursor_src_file`: 鼠标指针文件路径列表, 用于导出文件
    - `destination_dirs`: [DestinationDirs] 字段, 用于声明 Windows 读取 INF 文件时获取需要复制鼠标指针到的路径
    - `wreg`: [Wreg] 字段, 用于声明 Windows 读取 INF 文件时立刻执行的配置鼠标指针应用操作
    - `scheme_reg`: [Scheme.Reg] 字段, 写入到注册表中的鼠标指针方案
    - `scheme_cur`: [Scheme.Cur] 字段, 保存需要复制的鼠标指针列表
    - `strings`: [Strings] 字段, 保存 INF 文件中需要的变量

    Args:
        cursor_name (str): 要导出的鼠标指针方案的名称
        custom_install_path (Path | None): 自定义鼠标指针安装文件
    Returns:
        (dict[str, str]): 鼠标指针的配置字典
    """
    config_dict: dict[str, str] = {}

    # 查找鼠标指针对应的方案信息
    cursor_paths_in_reg = registry_query_value(
        name=cursor_name,
        sub_key=WINDOWS_CURSOR_CURSORS_SCHEME_PATH,
        key=RegistryRootKey.CURRENT_USER,
        access=RegistryAccess.READ,
    )

    cursor_paths: list[Path] = []  # 用于导出的路径列表
    paths_to_reg: list[str] = []  # [Scheme.Reg] 部分
    strings: dict[str, str] = {}  # [Strings] 部分
    wreg_list: list[str] = []  # [Wreg] 部分
    wreg_list.append(r'HKCU,"Control Panel\Cursors",,0x00020000,"%SCHEME_NAME%"')
    strings["SCHEME_NAME"] = cursor_name

    # 将路径字符串解释成实际鼠标指针文件路径
    raw_paths: list[str] = extend_list_to_length(cursor_paths_in_reg.split(","), target_length=len(CURSOR_KEYS["win"]))[: len(CURSOR_KEYS["win"])]
    for key, origin_path in zip(CURSOR_KEYS["win"], raw_paths):
        path = Path(expand_var_string(origin_path)) if origin_path.strip() != "" and Path(expand_var_string(origin_path)).is_file() else None
        if path is not None:
            cursor_paths.append(path)

        path_in_reg = ""
        if custom_install_path is not None and path is not None:
            # 使用自定义安装路径
            path_in_reg = str(custom_install_path / cursor_name / f"%{key}%")
        elif path is not None:
            # 使用默认安装路径
            path_in_reg = rf"%10%\%CUR_DIR%\%{key}%"

        paths_to_reg.append(path_in_reg)
        if path_in_reg != "":
            strings[key] = path.name
            wreg_list.append(rf'HKCU,"Control Panel\Cursors",{key},0x00020000,"{path_in_reg}"')

    wreg_list.append(r'HKLM,"SOFTWARE\Microsoft\Windows\CurrentVersion\Runonce\Setup\","",,"rundll32.exe shell32.dll,Control_RunDLL main.cpl @0"')
    wreg = "\n".join(wreg_list)

    # 配置 [DistinationDirs] 字段和部分 [Strings] 字段
    if custom_install_path is not None:
        # 使用自定义安装路径
        custom_path = str(custom_install_path / cursor_name)
        strings["CUR_DIR"] = custom_path
        destination_dirs = r'-1,"%CUR_DIR%"'
    else:
        strings["CUR_DIR"] = rf"Cursors\{cursor_name}"
        destination_dirs = r'10,"%CUR_DIR%"'

    # 配置 [Scheme.Reg] 字段
    paths_to_reg_string = ",".join(paths_to_reg)
    if custom_install_path is not None:
        scheme_reg = rf'HKCU,"Control Panel\Cursors\Schemes","%SCHEME_NAME%",,"{paths_to_reg_string}"'
    else:
        scheme_reg = rf'HKCU,"Control Panel\Cursors\Schemes","%SCHEME_NAME%",0x00020000,"{paths_to_reg_string}"'

    # 配置 [Scheme.Cur] 字段
    scheme_cur = "\n".join([f'"{x.name}"' for x in cursor_paths])

    # 打包参数
    config_dict["cursor_src_file"] = cursor_paths
    config_dict["destination_dirs"] = destination_dirs
    config_dict["wreg"] = wreg
    config_dict["scheme_reg"] = scheme_reg
    config_dict["scheme_cur"] = scheme_cur
    config_dict["strings"] = dict_to_inf_strings_format(strings)

    return config_dict



"""X.Org 环境配置工具"""

import re
from pathlib import Path

from ani2xcur.utils import safe_convert_to_int

X_RESOURCES_PATH = Path("~/.Xresources").expanduser()
"""X resources 配置文件路径"""


def read_x_resources_config(
    config_path: Path,
) -> dict[str, str]:
    """读取 X resources 配置文件并返回配置字典

    Args:
        config_path (Path): X resources 配置文件路径
    Returns:
        (dict[str, str]): X resources 配置字典
    """
    config = {}

    with open(config_path, "r", encoding="utf-8") as file:
        for line in file:
            line = line.strip()
            # 跳过注释行和空行
            if line.startswith("#") or not line:
                continue

            # 匹配格式: key: value
            match = re.match(r"^([^:]+):\s*(.*)$", line)
            if match:
                key, value = match.groups()
                key = key.strip()
                value = value.strip()

                config[key] = value

    return config


def write_x_resources_config(
    config_path: Path,
    updates: dict[str, str],
) -> None:
    """修改 X resources 配置文件

    Args:
        config_path (Path): 配置文件路径
        updates (dict[str, str]): 要更新的内容字典
    """
    lines = []
    if config_path.exists():
        with open(config_path, "r", encoding="utf-8") as f:
            lines = f.readlines()

    new_lines = []
    found_keys = set()

    # 遍历现有行进行更新
    for line in lines:
        stripped = line.strip()
        # Xresources 注释通常以 '!' 开头，但也兼容 '#'
        if not stripped or stripped.startswith("!") or stripped.startswith("#"):
            new_lines.append(line)
            continue

        match = re.match(r"^([^:]+):", stripped)
        if match:
            key = match.group(1).strip()
            if key in updates:
                new_lines.append(f"{key}: {updates[key]}\n")
                found_keys.add(key)
                continue

        new_lines.append(line)

    # 追加新配置项
    for key, val in updates.items():
        if key not in found_keys:
            new_lines.append(f"{key}: {val}\n")

    with open(config_path, "w", encoding="utf-8") as f:
        f.writelines(new_lines)


def get_x_resources_cursor_theme() -> str | None:
    """获取 X11 标准方案的当前使用的鼠标指针配置名称

    Returns:
        (str | None): 当前使用的鼠标指针名称
    """
    if not X_RESOURCES_PATH.is_file():
        return None

    config = read_x_resources_config(X_RESOURCES_PATH)
    return config.get("Xcursor.theme")


def get_x_resources_cursor_size() -> int | None:
    """获取 X11 标准方案的当前使用的鼠标指针大小

    Returns:
        (int | None): 当前使用的鼠标指针大小
    """
    if not X_RESOURCES_PATH.is_file():
        return None

    config = read_x_resources_config(X_RESOURCES_PATH)
    return safe_convert_to_int(config.get("Xcursor.size"))


def set_x_resources_cursor_theme(
    cursor_name: str,
) -> None:
    """设置 X11 标准方案的当前使用的鼠标指针配置名称

    Args:
        cursor_name (str): 要设置的鼠标指针配置名称
    """
    X_RESOURCES_PATH.parent.mkdir(parents=True, exist_ok=True)
    write_x_resources_config(config_path=X_RESOURCES_PATH, updates={"Xcursor.theme": cursor_name})


def set_x_resources_cursor_size(
    cursor_size: int,
) -> None:
    """设置 X11 标准方案的当前使用的鼠标指针大小

    Args:
        cursor_size (int): 要设置的鼠标指针大小
    """
    X_RESOURCES_PATH.parent.mkdir(parents=True, exist_ok=True)
    write_x_resources_config(config_path=X_RESOURCES_PATH, updates={"Xcursor.size": str(cursor_size)})
